 /*
 * File:		dac12bit_demo.c
 * Purpose:		Main process
 * TWR-K60F120M, Rev. AX2: User can probe DAC0_OUT and DAC1_OUT signals from Primary elevator express board at A32
 * 						   and B32 correspondingly.
 * TWR-K70FN1M ,Rev. A.: User can probe DAC0_OUT and DAC1_OUT signals from Primary elevator express board at A32
 * 						 and B32 correspondingly.
 * 
 * Rev 1 15 OCT 2010
 *
 */

#include "common.h"
#include "dac12bit.h"
/********************************************************************/
void main (void)
{
	char ch ='0';
        dac0_1_clk_enable();  //enabled system clock to DAC module.
        
  	printf("\nWelcome to the Kinetis 12-bit DAC module DEMOs. \n");      
        printf("\nPlease select the following option for the specfific demo\n\n"); 
   
        printf("\n 1: Show case selectable reference voltage capability\n");    
        printf("\n 2: Show case the DAC module capability when configured as buffered mode \n");    
        printf("\n 3: Same configuration from DEMO 2 except we are running in SWING mode \n");    
        printf("\n 4: Same configuration from DEMO 3 except we are runing in ONE TIME mode \n");  
        printf("\n 5: Show case DAC hardware triggered by the PDB module\n");   
        
        printf("\nYou will need to restart the application everytime you finish each demo\n"); 	
       
       
       ch = in_char();
        
	while(1)
	{
         
                      
          switch(ch){
          
          case '1':  
            /*  
            DEMO 1:
            This demo showcases DAC module's capability to select an input reference voltage from either  
            1 ) The external VDDA pin 
            2 ) The internal on-chip fixed reference voltage from the output of the VREF module.
            You will observe at each of the DAC0 and DAC1 pin, a repeated ramp linear signal will be generated. 
            The peak/max of the ramp signal generated by DAC0 or DAC1 is restricted by the corresponding input voltage reference value. 
            Since DAC0 selects at a lower input voltage reference from VREF output at about 1.2V(varies from part to part) and DAC1 selects a higher input voltage
            reference from VDDA pin at about 3.3V, the peak of DAC0 signal will be less than the DAC1 signal
            Notice Demo 1 sets up DAC modules in non-buffer mode. This means that DAC output solely base on the 
              digital value stores at DAC_DAT0 register. As in buffered mode, the DAC output value is base on the DAC_DATx value where the read pointer
            points to.
            */
            printf("\n You are in DEMO 1\n"); 
            printf("\n This is non-buffer mode where only the first data register is used.\n"); 
            printf("\n The SAW wave is made up of 4096 data points\n");   
            printf("\n You should expect DAC0_OUT to output a SAW wave at about VREFO(1.2V) amplitude\n");    
            printf("\n You should expect DAC1_OUT to output a SAW wave at about VDDA amplitude\n");    
            printf("\n You will need to reset in order to run other demo"); 	
            
            ch ='0';
            DACx_reset_dac0_1_reg_values();//reinitialize DAC0 and DAC1 registers to default reset values;
            DAC12_VreferenceInit(DAC0_BASE_PTR, DAC_SEL_VREFO);//DAC0 initialized as: Non-buffer mode and select VREF Output as reference  
            DAC12_VreferenceInit(DAC1_BASE_PTR, DAC_SEL_VDDA);//DAC1 initialized as: Non-buffer mode and select VDDA as reference 
            DAC12_VreferenceRamp(); //Ramp dac output from 0 to 4095 <=> DAC output = input reference voltage/4096 to input reference voltage(See datasheet for output restriction)
            
            
         
          break;
           
          case '2':
            /*DEMO 2: 
            This demo showcases the DAC module capability when configured as buffered mode
            When DAC is enabled buffered mode, the DAC output value is base on the DAC_DATx 
            value where the read pointer points to. There are totally 16 word DAC buffer(DAC_DAT0 to DAC_DAT16) for each
            DAC module. There are two configuration that you can choose to advance the read pointer to the next
            buffer. One is Software trigger another is hardware trigger by PDB timmer module. Also, in buffer mode
            You can define where the read pointer goes to after reading the top limit pointer.
            we will show case Software trigger and Normal mode.
           */
             
            printf("\n You are in DEMO 2\n"); 
            printf("\n In buffered mode where all the 16 word buffers are used \n"); 
            printf("\n to pre-load SAW wave output\n"); 
            printf("\n Only 16 data points were used to generate a SAW wave\n"); 
            printf("\n so the saw frequency is faster and steps are more visible compare to DEMO 1\n"); 
            printf("\n You should expect DAC0_OUT to output a SAW wave at about VREFO(1.2V) amplitude\n");    
            printf("\n You should expect DAC1_OUT to output a SAW wave at about VDDA amplitude\n");    
            printf("\n You will need to reset in order to run other demo"); 	
            
            ch ='0';
            DACx_reset_dac0_1_reg_values();//reset DAC0 and DAC1 value to default reset value;  
            DAC12_SoftTrigBuffInit(DAC0_BASE_PTR, DAC_BF_NORMAL_MODE, DAC_SEL_VREFO,DAC_SEL_SW_TRIG,DAC_SET_PTR_AT_BF(0),DAC_SET_PTR_UP_LIMIT(15) ) ; 
            DAC12_SoftTrigBuffInit(DAC1_BASE_PTR, DAC_BF_NORMAL_MODE, DAC_SEL_VDDA,DAC_SEL_SW_TRIG,DAC_SET_PTR_AT_BF(0),DAC_SET_PTR_UP_LIMIT(15) ) ; 
            DAC12_SoftwareTriggerLoop();
          
           
           break;
           
          case '3':
            /*DEMO 3
             Same configuration from DEMO 2 except we are running in SWING mode.
             Observe DACx output, you will notice the ramp will go down after it ramp up.
            */
           
            printf("\n You are in DEMO 3\n"); 
            printf("\n Same configuration from DEMO 2 except we are running in SWING mode.\n");    
            printf("\n DACx output will ramp up and down to generate a triangular waveform\n");     
            printf("\nYou will need to reset in order to run other demo"); 	  
            
            ch ='0'; 
            DACx_reset_dac0_1_reg_values();//reset DAC0 and DAC1 value to default reset value;  
            DAC12_SoftTrigBuffInit(DAC0_BASE_PTR, DAC_BF_SWING_MODE, DAC_SEL_VREFO,DAC_SEL_SW_TRIG,DAC_SET_PTR_AT_BF(0),DAC_SET_PTR_UP_LIMIT(15) ) ; 
            DAC12_SoftTrigBuffInit(DAC1_BASE_PTR, DAC_BF_SWING_MODE, DAC_SEL_VDDA,DAC_SEL_SW_TRIG,DAC_SET_PTR_AT_BF(0),DAC_SET_PTR_UP_LIMIT(15) ) ; 
            DAC12_SoftwareTriggerLoop();
             
            
            break;
          
          case '4':
            /*DEMO 4
             Same configuration from DEMO 3 except we are runing in ONE_TIME mode. 
             Observe DACx output, you will notice the ramp up will only occur once and won't repeat.
            */
           
            printf("\n You are in DEMO 4\n"); 
            printf("\n Same configuration from DEMO 2 except we are running in ONE TIME mode.\n");    
            printf("\n DACx output will ramp up only once and and stay high. It won't repeat\n");  
            printf("\nYou will need to reset in order to run other demo"); 		  
            
            ch ='0'; 
            DACx_reset_dac0_1_reg_values();//reset DAC0 and DAC1 value to default reset value;  
            DAC12_SoftTrigBuffInit(DAC0_BASE_PTR, DAC_BF_ONE_TIME_MODE, DAC_SEL_VREFO,DAC_SEL_SW_TRIG,DAC_SET_PTR_AT_BF(0),DAC_SET_PTR_UP_LIMIT(15) ) ; 
            DAC12_SoftTrigBuffInit(DAC1_BASE_PTR, DAC_BF_ONE_TIME_MODE, DAC_SEL_VDDA,DAC_SEL_SW_TRIG,DAC_SET_PTR_AT_BF(0),DAC_SET_PTR_UP_LIMIT(15) ) ; 
            DAC12_SoftwareTriggerLoop(); 
           
          
          break;
          
           case '5':
          /*DEMO 5
           This is an example of using hardware trigger by the PDB module, to advance to the next 
           read pointer in buffer mode. You will notice DAC output changes intervals are defined by the 
           value specified in the PDB_DACINTx register. See PDB_DAC0_TriggerInit or PDB_DAC1_TriggerInit
           for detail.
           */
  
             printf("\n You are in DEMO 5\n"); 
             printf("\n DACx output is done by hardware trigger via the PDB module.\n");    
             printf("\n read buffer pointer is advanced by time period set forth by the   \n");  
             printf("\n the value specified in the PDB_DACINTx register  \n");  
             printf("\nYou will need to reset in order to run other demo"); 	
             
             ch ='0';
             DACx_reset_dac0_1_reg_values();//reset DAC0 and DAC1 value to default reset value;  
             DAC12_HWTrigBuff(DAC0_BASE_PTR, DAC_BF_SWING_MODE,DAC_SEL_VREFO,DAC_SEL_PDB_HW_TRIG,DAC_SET_PTR_AT_BF(0),DAC_SET_PTR_UP_LIMIT(15)) ;  
             DAC12_HWTrigBuff(DAC1_BASE_PTR, DAC_BF_SWING_MODE,DAC_SEL_VDDA,DAC_SEL_PDB_HW_TRIG,DAC_SET_PTR_AT_BF(0),DAC_SET_PTR_UP_LIMIT(15)) ;  
  
            break;
          
     
           
           default:

            break;
          
          
          
          }  //end of switch statement    
                
	} 
        
        
}
/********************************************************************/
