/*
 * File:		dryice.c
 * Purpose:		Main process
 *
 */

#include "common.h"
#include "dryice_functions.h"
#include "smc.h"
#include "uart.h"

#define ENABLE  1
#define DISABLE 0


//Tamper Pins
#define TAMPER_PIN0 0
#define TAMPER_PIN1 1
#define TAMPER_PIN2 2
#define TAMPER_PIN3 3
#define TAMPER_PIN4 4
#define TAMPER_PIN5 5
#define TAMPER_PIN6 6
#define TAMPER_PIN7 7

//Glitch Filter Clock Source
#define PRESCALER_CLK_512_HZ 0
#define PRESCALER_CLK_32_KHZ 1

//Tamper Pin Polarity
#define NOT_INVERTED 0
#define INVERTED     1

//Tamper Pin Direction
#define INPUT                  0
#define OUTPUT_INVERSE_EXP_VAL 1

//Tamper Pin Expected Values
#define LOGIC_ZERO                0
#define ACTIVE_TAMPER_0_OUTPUT    1
#define ACTIVE_TAMPER_1_OUTPUT    2
#define XOR_ACTIVE_TAMPER_OUTPUTs 3

//Active Tamper defines
#define ACTIVE_TAMPER_0 0
#define ACTIVE_TAMPER_1 1

//Internal Tamper Sources
#define DRYICE_TAMPER           0
#define TIME_OVERFLOW           2
#define MONOTONIC_OVERFLOW      3
#define VBAT_TAMPER             4
#define RTCOSC_TAMPER           5
#define TEMPERATURE_TAMPER      6
#define FLASH_SECURITY_DISABLED 7
#define TEST_MODE_ENTRY         8

/********************************************************************/
void main (void)
{
        int active_tamper_polynomial = 0xAADD;
        int active_tamper_shift_reg = 0x5522;
                
  	printf("\nWelcome to the DryIce demo.\n");

        dryice_init();
        dryice_enable_clock_and_prescaler(ENABLE);
        dryice_active_tamper_config(ACTIVE_TAMPER_0,active_tamper_polynomial,active_tamper_shift_reg);
        dryice_tamper_pin_expect_config(TAMPER_PIN1,ACTIVE_TAMPER_0_OUTPUT);
        dryice_tamper_pin_polarity_config(TAMPER_PIN1,INVERTED);
        dryice_tamper_pin_direction_config(TAMPER_PIN1,OUTPUT_INVERSE_EXP_VAL);
        
        dryice_tamper_pin_expect_config(TAMPER_PIN2,ACTIVE_TAMPER_0_OUTPUT);
        dryice_tamper_pin_pull_config(TAMPER_PIN2,ENABLE);
        glitch_filter_width_select(TAMPER_PIN2,10);
        glitch_filter_prescaler_select(TAMPER_PIN2,PRESCALER_CLK_512_HZ);
        glitch_filter_prescaler_switch(TAMPER_PIN2,ENABLE);
//        dryice_pin_tamper_interrupt_config(TAMPER_PIN2,ENABLE);
        dryice_pin_tamper_config(TAMPER_PIN2,ENABLE);
        dryice_int_tamper_interrupt_config(DRYICE_TAMPER,ENABLE);
        // Enable IRQ 99 for DryIce interrupts
        enable_irq(99); 
        EnableInterrupts;
        printf("The active tamper has been configured.\n");
        printf("\nDisconnect jumper J21 from the TWR-K70F120M board to simulate a tamper between\n");
        printf("Tamper Pin 1 and Tamper Pin 2. Once the jumper is removed, a DryIce interrupt will occur.\n");
        while(1);

}
/********************************************************************/
void dryice_init(void)
{
        int32 i;
        int rtc_tsr_reg;
        DisableInterrupts;
        
        /* Enable clocks to DryIce Secure Registers and to DryIce  Module */
        SIM_SCGC5|=SIM_SCGC5_DRYICESECREG_MASK | SIM_SCGC5_DRYICE_MASK;
        /* Enable clocks to RTC Module */
        SIM_SCGC6|=SIM_SCGC6_RTC_MASK;
        /* Enable EXTAL32 */
        RTC_CR |= RTC_CR_OSCE_MASK;
        
        /* Wait awhile for EXTAL32 to start up */
        for(i=0;i<0x00FFFFFF;i++) {
            asm("nop");
        }
        
        /* Use EXTAL32/RTCOSC (not OSC032KCLK) to source ERCLK32K*/
        SIM_SOPT1 &= ~SIM_SOPT1_OSC32KSEL_MASK;
        
        /* Output 32 kHz clock to other peripherals */
        RTC_CR &= ~RTC_CR_CLKO_MASK;

      	/* Enable the CLKOUT function on PTC3 (alt5 function)*/
        PORTC_PCR3 = (PORT_PCR_MUX(0x5));

        /* Drive RTC oscillator out on PTC3 */
        SIM_SOPT2 |= SIM_SOPT2_CLKOUTSEL(5);
        
        /*Set time compensation parameters*/
        RTC_TCR = RTC_TCR_CIR(0) | RTC_TCR_TCR(0);
        
        rtc_tsr_reg = RTC_TSR;

        if (rtc_tsr_reg < 0x100)
        {
          /*Initialize the timer seconds register*/
          RTC_TSR = 0x00000100;
          
          /*Enable the RTC time counter*/
          RTC_SR |= RTC_SR_TCE_MASK;        
        }
          
       
        //Clear DTF
        DRY_SR |= DRY_SR_DTF_MASK;
        //Clear TAF
        DRY_SR |= DRY_SR_TAF_MASK;
        
        if ((FTFE_FSEC & 0x3) == 0x2)
        {
          printf("Flash is unsecured.\n\n");
        }
        else
        {
          printf("Flash is secured.\n\n");
        }
}
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
void dryice_isr(void) 
{
   printf("\n\n============================================================\n");  
   printf("                     Entered DryIce ISR\n");
   printf("============================================================\n");   
   
   if((((DRY_SR & DRY_SR_TOF_MASK) == 0x04) && (DRY_IER & DRY_IER_TOIE_MASK)==DRY_IER_TOIE_MASK))
   {
     printf("DryIce Time Overflow in RTC detected...\r\n");
     printf("Disabling RTC time counter.\n");
     RTC_SR &= ~RTC_SR_TCE_MASK;
     printf("Writing to RTC_TSR to clear RTC_SR[TOF].\n");
     RTC_TSR |= 0xAAAA5555;
     printf("Writing to DRY_SR[TOF] to clear DRY_SR[TOF].\n");
     DRY_SR |= DRY_SR_TOF_MASK;
     printf("DRY_SR is %08X\n",DRY_SR);
   }	 	
   if((((DRY_SR & DRY_SR_MOF_MASK) == 0x08)&& (DRY_IER & DRY_IER_MOIE_MASK)==DRY_IER_MOIE_MASK))
   {
     printf("DryIce Monotonic Overflow in RTC detected...\r\n");
     printf("Disabling RTC monotonic counter.\n");           
     RTC_MER &= ~RTC_MER_MCE_MASK; // Disable monotonic counter
     printf("Writing to RTC_MCHR to clear RTC_SR[MOF].\n");           
     RTC_MCHR |= 0xFFFFFFFF; // Write to monotonic counter register high
     printf("Writing to DRY_SR[MOF] to clear DRY_SR[MOF].\n");
     DRY_SR |= DRY_SR_MOF_MASK;
     printf("DRY_SR is %08X\n",DRY_SR);
   }	
   if((((DRY_SR & DRY_SR_VTF_MASK) == 0x10)&& (DRY_IER & DRY_IER_VTIE_MASK)==DRY_IER_VTIE_MASK))
   {
     printf("DryIce Voltage Tamper detected...\r\n");
     printf("Please bring VBAT voltage to within the acceptable range and press any key to continue\n");
     // Wait to ensure VBAT is within proper range first                      
     in_char();           
     printf("Clearing VTF by writing 1 to it.\n");
     DRY_SR |= DRY_SR_VTF_MASK; // clear VTF
     printf("DRY_SR is %08X\n",DRY_SR);           
    }	
   if((((DRY_SR & DRY_SR_CTF_MASK) == 0x20)&& (DRY_IER & DRY_IER_CTIE_MASK)==DRY_IER_CTIE_MASK))
   {
     printf("DryIce Clock Tamper detected...\r\n");
     printf("Please bring EXTAL32 clock frequency to within the acceptable range and press any key to continue\n");
     // Ensure clock is within proper range first
     in_char();                      
     printf("Clearing CTF by writing 1 to it.\n");
     DRY_SR |= DRY_SR_CTF_MASK; // clear CTF
     printf("DRY_SR is %08X\n",DRY_SR);                      
   }	
   if((((DRY_SR & DRY_SR_TTF_MASK) == 0x40)&& (DRY_IER & DRY_IER_TTIE_MASK)==DRY_IER_TTIE_MASK))
   {
     printf("DryIce Temperature Tamper detected...\r\n");
     printf("Please bring temperature to within the acceptable range and press any key to continue\n");           
     // Ensure temperature is within proper range first
     in_char();                                 
     printf("Clearing TTF by writing 1 to it.\n");           
     DRY_SR |= DRY_SR_TTF_MASK; // clear TTF
     printf("DRY_SR is %08X\n",DRY_SR);                      
   }
   if((((DRY_SR & DRY_SR_FSF_MASK) == 0x100)&& (DRY_IER & DRY_IER_FSIE_MASK)==DRY_IER_FSIE_MASK))
   {
     printf("DryIce Flash Security Tamper detected...\r\n");
     printf("Flash security byte is: %08X\n",FTFE_FSEC);
     // Ensure flash security has been enabled first
     if ((FTFE_FSEC & 0x2) == 0x2)
     {
       printf("Flash is unsecured. Cannot clear FSF bit. Need to secure the flash first before \n");
       printf("Please issue a reset and unsecure the flash.\n");
     }
     else
     {
       printf("Flash is secured. Attempting to clear FSF bit.\n");
       DRY_SR |= DRY_SR_FSF_MASK; // clear FSF       
     }
   }	
   if((((DRY_SR & DRY_SR_TMF_MASK) == 0x200)&& (DRY_IER & DRY_IER_TMIE_MASK)==DRY_IER_TMIE_MASK))
   {
     printf("DryIce Test Mode detected...\r\n");
     printf("DryIce Test Mode must be exited before TMF can be cleared.\n");
     // Ensure test mode has been exited first
     printf("Exit test mode and enter any key to continue.\n");
     in_char();
     DRY_SR &= DRY_SR_TMF_MASK; // clear VTF
   }	
   if((((DRY_SR & DRY_SR_TPF(0x1)) == DRY_SR_TPF(0x1))&& (DRY_IER & DRY_IER_TPIE(0x1))==DRY_IER_TPIE(0x1)))
   {
     printf("DryIce Tamper Pin 0 tamper event detected...\r\n");
     // Ensure tamper pin equals its expected value
     printf("Change Tamper Pin 0 to its expected value and press any key to continue.\n");
     in_char();
     DRY_SR |= DRY_SR_TPF(0x1); // clear TPF for associated pin
   }	
   if((((DRY_SR & DRY_SR_TPF(0x2)) == DRY_SR_TPF(0x2))&& (DRY_IER & DRY_IER_TPIE(0x2))==DRY_IER_TPIE(0x2)))
   {
     printf("DryIce Tamper Pin 1 tamper event detected...\r\n");
     // Ensure tamper pin equals its expected value
     printf("Change Tamper Pin 1 to its expected value and press any key to continue.\n");
     in_char();           
     DRY_SR |= DRY_SR_TPF(0x02); // clear TPF for associated pin
     
   }	
   if ((((DRY_SR & DRY_SR_TPF(0x4)) == DRY_SR_TPF(0x4))&& (DRY_IER & DRY_IER_TPIE(0x4))==DRY_IER_TPIE(0x4)))
   {
     printf("DryIce Tamper Pin 2 tamper event detected...\r\n");
     // Ensure tamper pin equals its expected value
     printf("Change Tamper Pin 2 to its expected value and press any key to continue.\n");
     in_char();                      
     DRY_SR |= DRY_SR_TPF(0x04); // clear TPF for associated pin
   }	
   if((((DRY_SR & DRY_SR_TPF(0x8)) == DRY_SR_TPF(0x8))&& (DRY_IER & DRY_IER_TPIE(0x8))==DRY_IER_TPIE(0x8)))
   {
     printf("DryIce Tamper Pin 3 tamper event detected...\r\n");
     // Ensure tamper pin equals its expected value
     printf("Change Tamper Pin 3 to its expected value and press any key to continue.\n");
     in_char();                      
     DRY_SR |= DRY_SR_TPF(0x08); // clear TPF for associated pin
   }	
   if((((DRY_SR & DRY_SR_TPF(0x10)) == DRY_SR_TPF(0x10))&& (DRY_IER & DRY_IER_TPIE(0x10))==DRY_IER_TPIE(0x10)))
   {
     printf("DryIce Tamper Pin 4 tamper event detected...\r\n");
     // Ensure tamper pin equals its expected value
     printf("Change Tamper Pin 4 to its expected value and press any key to continue.\n");
     in_char();                      
     DRY_SR |= DRY_SR_TPF(0x10); // clear TPF for associated pin
   }	
   if((((DRY_SR & DRY_SR_TPF(0x20)) == DRY_SR_TPF(0x20))&& (DRY_IER & DRY_IER_TPIE(0x20))==DRY_IER_TPIE(0x20)))
   {
     printf("DryIce Tamper Pin 5 tamper event detected...\r\n");
     // Ensure tamper pin equals its expected value
     printf("Change Tamper Pin 5 to its expected value and press any key to continue.\n");
     in_char();                      
     DRY_SR |= DRY_SR_TPF(0x20); // clear TPF for associated pin
   }	
   if((((DRY_SR & DRY_SR_TPF(0x40)) == DRY_SR_TPF(0x40))&& (DRY_IER & DRY_IER_TPIE(0x40))==DRY_IER_TPIE(0x40)))
   {
     printf("DryIce Tamper Pin 6 tamper event detected...\r\n");
     // Ensure tamper pin equals its expected value
     printf("Change Tamper Pin 6 to its expected value and press any key to continue.\n");
     in_char();                      
     DRY_SR |= DRY_SR_TPF(0x40); // clear TPF for associated pin
   }	
   if((((DRY_SR & DRY_SR_TPF(0x80)) == DRY_SR_TPF(0x80))&& (DRY_IER & DRY_IER_TPIE(0x80))==DRY_IER_TPIE(0x80)))
   {
     printf("DryIce Tamper Pin 7 tamper event detected...\r\n");
     // Ensure tamper pin equals its expected value
     printf("Change Tamper Pin 7 to its expected value and press any key to continue.\n");
     in_char();                      
     DRY_SR |= DRY_SR_TPF(0x80); // clear TPF for associated pin
   }	
   if(((DRY_SR & DRY_SR_DTF_MASK)== DRY_SR_DTF_MASK) && ((DRY_IER & DRY_IER_DTIE_MASK) == DRY_IER_DTIE_MASK))
   {
     printf("DryIce Tamper Flag tamper event detected...\r\n");
     printf("\nDRY_TSR is %08X\n\n",DRY_TSR);
     
     if((((DRY_SR & DRY_SR_TOF_MASK) == 0x04) && (DRY_TER & DRY_TER_TOE_MASK)==DRY_TER_TOE_MASK))
     {
       printf("DryIce Time Overflow in RTC detected...\r\n");
       printf("Disabling RTC time counter.\n");
       RTC_SR &= ~RTC_SR_TCE_MASK;
       printf("Writing to RTC_TSR to clear RTC_SR[TOF].\n");
       RTC_TSR |= 0xAAAA5555;
       printf("Writing to DRY_SR[TOF] to clear DRY_SR[TOF].\n");
       DRY_SR |= DRY_SR_TOF_MASK;
       printf("DRY_SR is %08X\n",DRY_SR);
     }	 	
     if((((DRY_SR & DRY_SR_MOF_MASK) == 0x08)&& (DRY_TER & DRY_TER_MOE_MASK)==DRY_TER_MOE_MASK))
     {
       printf("DryIce Monotonic Overflow in RTC detected...\r\n");
       printf("Disabling RTC monotonic counter.\n");           
       RTC_MER &= ~RTC_MER_MCE_MASK; // Disable monotonic counter
       printf("Writing to RTC_MCHR to clear RTC_SR[MOF].\n");           
       RTC_MCHR |= 0xFFFFFFFF; // Write to monotonic counter register high
       printf("Writing to DRY_SR[MOF] to clear DRY_SR[MOF].\n");
       DRY_SR |= DRY_SR_MOF_MASK;
       printf("DRY_SR is %08X\n",DRY_SR);
     }	
     if((((DRY_SR & DRY_SR_VTF_MASK) == 0x10)&& (DRY_TER & DRY_TER_VTE_MASK)==DRY_TER_VTE_MASK))
     {
       printf("DryIce Voltage Tamper detected...\r\n");
       printf("Please bring VBAT voltage to within the acceptable range and press any key to continue\n");
       // Wait to ensure VBAT is within proper range first                      
       in_char();           
       printf("Clearing VTF by writing 1 to it.\n");
       DRY_SR |= DRY_SR_VTF_MASK; // clear VTF
       printf("DRY_SR is %08X\n",DRY_SR);           
     }	
     if((((DRY_SR & DRY_SR_CTF_MASK) == 0x20)&& (DRY_TER & DRY_TER_CTE_MASK)==DRY_TER_CTE_MASK))
     {
       printf("DryIce Clock Tamper detected...\r\n");
       printf("Please bring EXTAL32 clock frequency to within the acceptable range and press any key to continue\n");
       // Ensure clock is within proper range first
       in_char();                      
       printf("Clearing CTF by writing 1 to it.\n");
       DRY_SR |= DRY_SR_CTF_MASK; // clear CTF
       printf("DRY_SR is %08X\n",DRY_SR);                      
     }	
     if((((DRY_SR & DRY_SR_TTF_MASK) == 0x40)&& (DRY_TER & DRY_TER_TTE_MASK)==DRY_TER_TTE_MASK))
     {
       printf("DryIce Temperature Tamper detected...\r\n");
       printf("Please bring temperature to within the acceptable range and press any key to continue\n");           
       // Ensure temperature is within proper range first
       in_char();                                 
       printf("Clearing TTF by writing 1 to it.\n");           
       DRY_SR |= DRY_SR_TTF_MASK; // clear TTF
       printf("DRY_SR is %08X\n",DRY_SR);                      
     }
     if((((DRY_SR & DRY_SR_FSF_MASK) == 0x100)&& (DRY_TER & DRY_TER_FSE_MASK)==DRY_TER_FSE_MASK))
     {
       printf("DryIce Flash Security Tamper detected...\r\n");
       printf("Flash security byte is: %08X\n",FTFE_FSEC);
       // Ensure flash security has been enabled first
       if ((FTFE_FSEC & 0x2) == 0x2)
       {
         printf("Flash is unsecured. Cannot clear FSF bit. Need to secure the flash first before \n");
         printf("Please issue a reset and unsecure the flash.\n");
       }
       else
       {
         printf("Flash is secured. Attempting to clear FSF bit.\n");
         DRY_SR |= DRY_SR_FSF_MASK; // clear FSF       
       }
     }	
     if((((DRY_SR & DRY_SR_TMF_MASK) == 0x200)&& (DRY_TER & DRY_TER_TME_MASK)==DRY_TER_TME_MASK))
     {
       printf("DryIce Test Mode detected...\r\n");
       printf("DryIce Test Mode must be exited before TMF can be cleared.\n");
       // Ensure test mode has been exited first
       printf("Exit test mode and enter any key to continue.\n");
       in_char();
       DRY_SR &= DRY_SR_TMF_MASK; // clear VTF
     }	
     if((((DRY_SR & DRY_SR_TPF(0x1)) == DRY_SR_TPF(0x1))&& (DRY_TER & DRY_TER_TPE(0x1))==DRY_TER_TPE(0x1)))
     {
       printf("DryIce Tamper Pin 0 tamper event detected...\r\n");
       // Ensure tamper pin equals its expected value
       printf("Change Tamper Pin 0 to its expected value and press any key to continue.\n");
       in_char();
       DRY_SR |= DRY_SR_TPF(0x1); // clear TPF for associated pin
     }	
     if((((DRY_SR & DRY_SR_TPF(0x2)) == DRY_SR_TPF(0x2))&& (DRY_TER & DRY_TER_TPE(0x2))==DRY_TER_TPE(0x2)))
     {
       printf("DryIce Tamper Pin 1 tamper event detected...\r\n");
       // Ensure tamper pin equals its expected value
       printf("Change Tamper Pin 1 to its expected value and press any key to continue.\n");
       in_char();           
       DRY_SR |= DRY_SR_TPF(0x02); // clear TPF for associated pin
       
     }	
     if ((((DRY_SR & DRY_SR_TPF(0x4)) == DRY_SR_TPF(0x4))&& (DRY_TER & DRY_TER_TPE(0x4))==DRY_TER_TPE(0x4)))
     {
       printf("DryIce Tamper Pin 2 tamper event detected...\r\n");
       // Ensure tamper pin equals its expected value
       printf("Change Tamper Pin 2 to its expected value and press any key to continue.\n");
       in_char();                      
       DRY_SR |= DRY_SR_TPF(0x04); // clear TPF for associated pin
     }	
     if((((DRY_SR & DRY_SR_TPF(0x8)) == DRY_SR_TPF(0x8))&& (DRY_TER & DRY_TER_TPE(0x8))==DRY_TER_TPE(0x8)))
     {
       printf("DryIce Tamper Pin 3 tamper event detected...\r\n");
       // Ensure tamper pin equals its expected value
       printf("Change Tamper Pin 3 to its expected value and press any key to continue.\n");
       in_char();                      
       DRY_SR |= DRY_SR_TPF(0x08); // clear TPF for associated pin
     }	
     if((((DRY_SR & DRY_SR_TPF(0x10)) == DRY_SR_TPF(0x10))&& (DRY_TER & DRY_TER_TPE(0x10))==DRY_TER_TPE(0x10)))
     {
       printf("DryIce Tamper Pin 4 tamper event detected...\r\n");
       // Ensure tamper pin equals its expected value
       printf("Change Tamper Pin 4 to its expected value and press any key to continue.\n");
       in_char();                      
       DRY_SR |= DRY_SR_TPF(0x10); // clear TPF for associated pin
     }	
     if((((DRY_SR & DRY_SR_TPF(0x20)) == DRY_SR_TPF(0x20))&& (DRY_TER & DRY_TER_TPE(0x20))==DRY_TER_TPE(0x20)))
     {
       printf("DryIce Tamper Pin 5 tamper event detected...\r\n");
       // Ensure tamper pin equals its expected value
       printf("Change Tamper Pin 5 to its expected value and press any key to continue.\n");
       in_char();                      
       DRY_SR |= DRY_SR_TPF(0x20); // clear TPF for associated pin
     }	
     if((((DRY_SR & DRY_SR_TPF(0x40)) == DRY_SR_TPF(0x40))&& (DRY_TER & DRY_TER_TPE(0x40))==DRY_TER_TPE(0x40)))
     {
       printf("DryIce Tamper Pin 6 tamper event detected...\r\n");
       // Ensure tamper pin equals its expected value
       printf("Change Tamper Pin 6 to its expected value and press any key to continue.\n");
       in_char();                      
       DRY_SR |= DRY_SR_TPF(0x40); // clear TPF for associated pin
     }	
     if((((DRY_SR & DRY_SR_TPF(0x80)) == DRY_SR_TPF(0x80))&& (DRY_TER & DRY_TER_TPE(0x80))==DRY_TER_TPE(0x80)))
     {
       printf("DryIce Tamper Pin 7 tamper event detected...\r\n");
       // Ensure tamper pin equals its expected value
       printf("Change Tamper Pin 7 to its expected value and press any key to continue.\n");
       in_char();                      
       DRY_SR |= DRY_SR_TPF(0x80); // clear TPF for associated pin
     }     
     
     if ((DRY_SR & (DRY_SR_TAF_MASK | DRY_SR_DTF_MASK)) == (DRY_SR_TAF_MASK | DRY_SR_DTF_MASK))
     {
       printf("DTF and TAF bits both set.\n");
       printf("Clearing DTF first.\n");
       DRY_SR |= DRY_SR_DTF_MASK;
       printf("DRY_SR is %08X\n\n",DRY_SR); 
       printf("Clearing TAF next.\n");
       DRY_SR |= DRY_SR_TAF_MASK;
       printf("DRY_SR is %08X\n\n",DRY_SR); 
       printf("Press any key to continue\n");
       in_char();             
     }
     else if((DRY_SR & DRY_SR_DTF_MASK) == DRY_SR_DTF_MASK)
     {
       printf("DTF bit is set but TAF bit is clear. To clear DTF, TAF must also be set. A chip reset must be performed to set TAF before DTF can be cleared.\n");
       printf("Please issue a chip reset to exit the DryIce ISR.\n");
       in_char();
     }
   }
 
   printf("\n============================================================\n");  
   printf("                     Exiting DryIce ISR\n");
   printf("============================================================\n");   
   return;
}