/*
 * File:		dspi.c
 * Purpose:		K70 Example Code for DSPI
 *
 */

#include "common.h"
#include "dspi_common.h"
#include "edma_common_dspi.h"
#include "pioneer3_edma.h"

/* defines */
#define CORE_SRAM_ADDR 0x20000000 

/* prototypes */
void dspi_init(void);                               		//initialize pin muxing for all three DSPI modules
uint32_t dspi_no_fifo_3(uint32_t master, uint32_t slave);   //send and receive data, no fifo enabled, input args:  DSPI0, DSPI1, or DSPI2
uint32_t dspi0_master_dspi1_slave_test_dma(uint8_t mux);	//use dma for master/slave		

/* globals */
dspi_ctl dspi, dspi_s;
dma_tcd  tcd,tcd1,tcd2,tcd3;
uint32 data_to_slave[0x100];
uint32 data_from_slave[0x100];
uint32 data_from_master[0x100];
uint32 data_to_master[0x100];
volatile struct	SPI_MemMap *ptrDSPI[NUM_DSPI] = {SPI0_BASE_PTR, SPI1_BASE_PTR, SPI2_BASE_PTR};

/********************************************************************/
void main (void)
{
	uint32_t status;
        
  	printf("\nRunning the dspi examples.\n");

  	status = dspi_no_fifo_3(DSPI1, DSPI0);                //no fifo operation
  	status = dspi0_master_dspi1_slave_test_dma(0);        //dma tx & rx.  pass in 0=mux0, 1=mux1
  	  	
  	printf("\nFinished running dspi examples.\n");
}
/********************************************************************/

/********************************************************************/
/* Init all ports for DSPI */
/********************************************************************/
void dspi_init(void) 
{
  /* clock gate */
  SIM_SCGC6 |= SIM_SCGC6_DSPI0_MASK|SIM_SCGC6_DSPI1_MASK;
  SIM_SCGC3 |= SIM_SCGC3_DSPI2_MASK;

  /* pin mux */
  PORTA_PCR14 &= ~PORT_PCR_MUX_MASK;
  PORTA_PCR14 |= PORT_PCR_MUX(2); //SPI0_PCS0   //PTC4, PTE16, PTD0
  PORTA_PCR15 &= ~PORT_PCR_MUX_MASK;
  PORTA_PCR15 |= PORT_PCR_MUX(2); //SPI0_SCK    //PTC5, PTD1, PTE17
  PORTA_PCR16 &= ~PORT_PCR_MUX_MASK;
  PORTA_PCR16 |= PORT_PCR_MUX(2); //SPI0_SOUT   //PTC6, PTD2, PTE18
  PORTA_PCR17 &= ~PORT_PCR_MUX_MASK;
  PORTA_PCR17 |= PORT_PCR_MUX(2); //SPI0_SIN    //PTC7, PTD3, PTE19
  
  PORTB_PCR10 &= ~PORT_PCR_MUX_MASK;
  PORTB_PCR10 |= PORT_PCR_MUX(2); //SPI1_PCS0   //PTE4 
  PORTB_PCR11 &= ~PORT_PCR_MUX_MASK;
  PORTB_PCR11 |= PORT_PCR_MUX(2); //SPI1_SCK    //PTE2
  PORTB_PCR16 &= ~PORT_PCR_MUX_MASK;
  PORTB_PCR16 |= PORT_PCR_MUX(2); //SPI1_SOUT   //PTE1, PTE3
  PORTB_PCR17 &= ~PORT_PCR_MUX_MASK;
  PORTB_PCR17 |= PORT_PCR_MUX(2); //SPI1_SIN    //PTE1, PTE3
  
  PORTD_PCR11 &= ~PORT_PCR_MUX_MASK;
  PORTD_PCR11 |= PORT_PCR_MUX(2); //SPI2_PCS0   //PTF16, PTB20 
  PORTD_PCR12 &= ~PORT_PCR_MUX_MASK;
  PORTD_PCR12 |= PORT_PCR_MUX(2); //SPI2_SCK    //PTF17, PTB21
  PORTD_PCR13 &= ~PORT_PCR_MUX_MASK;
  PORTD_PCR13 |= PORT_PCR_MUX(2); //SPI2_SOUT   //PTF18, PTB22
  PORTD_PCR14 &= ~PORT_PCR_MUX_MASK;
  PORTD_PCR14 |= PORT_PCR_MUX(2); //SPI2_SIN    //PTF19, PTB23
}

uint32_t dspi_no_fifo_3(uint32_t master, uint32_t slave)
{
	uint32 mstr_pop_reg, slv_pop_reg, error_count = 0;
	
	dspi.txdata = 0xA8A9;
	dspi.br     = 0x00;
	dspi.cpha   = 0;
	dspi.cpol   = 0;
	
	ptrDSPI[master]->MCR = 0x80000C01; // flush fifo
	ptrDSPI[slave]->MCR = 0x00000c01;  
	
	printf("\r\nDSPI No FIFO configuration...\r\n");
	
	/********************************************************************************/
	/********************************************************************************/
	/* Use Scope to capture signals with different attributes to test DSPI accuracy */
	/********************************************************************************/
	/********************************************************************************/  
	ptrDSPI[master]->MCR = 0x80013001;              // set DIS_TXF and DIS_RXF to disable FIFO
	ptrDSPI[slave]->MCR = 0x00010001;               // slave, mdis=0, halt=1.  
	
	ptrDSPI[master]->CTAR[0] = 0x78000000;          // can use 0x78000008 - divide by 256
	ptrDSPI[slave]->CTAR_SLAVE[0] = 0x78000000;     // same as master
	
	ptrDSPI[slave]->MCR &= 0xFFFFFFFE;                              // Clear HALT bit
	ptrDSPI[master]->MCR &= 0xFFFFFFFE;                             // Clear HALT bit

	/* Make sure to have master connected to slave */    
	
	/* Send Data */  
	ptrDSPI[slave]->PUSHR = (0x0001AAAA);                               // slave data
	
	//ptrDSPI[master]->PUSHR = (0x00010000 | (dspi.txdata & 0xFFFF));     // send data with CONT=0, CTAS=0   
	ptrDSPI[master]->PUSHR = (0x08010000 | (dspi.txdata & 0xFFFF));   // send data with CONT=0, CTAS=0, EOQ=1

	while((ptrDSPI[master]->SR & SPI_SR_TCF) == 0){;}  
	ptrDSPI[master]->SR = ptrDSPI[master]->SR | SPI_SR_TCF ; 
		
	/* halt */
	ptrDSPI[slave]->MCR |= 1;                              // set HALT bit
	ptrDSPI[master]->MCR |= 1;                             // set HALT bit 
	
	/* read data */
	slv_pop_reg = ptrDSPI[slave]->POPR ;    
	mstr_pop_reg = ptrDSPI[master]->POPR ;    
	
	/* compare sent data with received data */
	if (slv_pop_reg != (dspi.txdata & 0xFFFF))
	{
		printf("Fail data transfer with data %x\n",dspi.txdata);
		error_count++;
	}
}

uint32_t dspi0_master_dspi1_slave_test_dma(uint8_t mux)
{
    s32 i, data= 0x80011234,data1= 0x80011122;
    int error_count = 0;
    for(i=0; i<0x100 ; i++,data++,data1++ )
    {
        data_to_slave[i] = data;
        data_from_slave[i] = 0xDEADDEAD;		
        data_to_master[i] = data1;
        data_from_master[i] = 0xDEADDEAD;
    }
    
    /* clear flags from any previous tests */
    SPI0_SR |= SPI0_SR ;
    SPI1_SR |= SPI1_SR ;
    SPI0_MCR |= 0x00000C01; 
    SPI1_MCR |= 0x00000C01;     
    
    data_to_slave[255] |= 0x08000000; //Setting EOQ bit
    data_to_slave[255] &= 0x7FFFFFFF;
    
    /* eDMA Configurations */
    //SIM_SCGC6 |= SIM_SCGC6_DMAMUX_MASK;
    SIM_SCGC6 |= (SIM_SCGC6_DMAMUX0_MASK | SIM_SCGC6_DMAMUX1_MASK); 
    
    if (mux == 0)
    {
      DMA_MUX0_CHCONFIG15 = 0x00; 
      DMA_MUX0_CHCONFIG14 = 0x00; 
      DMA_MUX0_CHCONFIG13 = 0x00; 
      DMA_MUX0_CHCONFIG12 = 0x00; 
      
      EDMA_ERQL = 0x0000F000;  // Enable DMA request 15,14,13,12
 
      DMA_MUX0_CHCONFIG15 = 0x93;  // SPI1 TX DMA request source enable 
      DMA_MUX0_CHCONFIG14 = 0x92;  // SPI1 RX DMA request source enable 
      DMA_MUX0_CHCONFIG13 = 0x91;  // SPI0 TX DMA request source enable 
      DMA_MUX0_CHCONFIG12 = 0x90;  // SPI0 RX DMA request source enable 
 
    }
    else
    {
      /* mux1 */
      DMA_MUX1_CHCONFIG15 = 0x00;  
      DMA_MUX1_CHCONFIG14 = 0x00; 
      DMA_MUX1_CHCONFIG13 = 0x00; 
      DMA_MUX1_CHCONFIG12 = 0x00; 
      
      EDMA_ERQL = 0xF0000000;  // Enable DMA request 31,30,29,28   
      
      DMA_MUX1_CHCONFIG15 = 0x93;  // SPI1 TX DMA request source enable 
      DMA_MUX1_CHCONFIG14 = 0x92;  // SPI1 RX DMA request source enable 
      DMA_MUX1_CHCONFIG13 = 0x91;  // SPI0 TX DMA request source enable 
      DMA_MUX1_CHCONFIG12 = 0x90;  // SPI0 RX DMA request source enable                 
    }
    
    
    // SPI0 TX
    tcd.saddr = (uint32)data_to_slave;
    tcd.daddr = (uint32)&SPI0_PUSHR;
    tcd.nbytes = 4;
    tcd.tcdAttr = EDMA_TCD_ATTR_SSIZE_32BIT|EDMA_TCD_ATTR_DSIZE_32BIT ;
    tcd.soff =0x04;
    tcd.doff =0x00;
    tcd.loopcount = 256;
    if (mux == 0)
      tcd.channelno = 13;
    else
      tcd.channelno = 29;
    
    dma_config(CONFIG_BASIC_XFR, &tcd);
    if (mux == 0)
      EDMA_TCD_CSR(13) = 0x08; //set DREQ
    else
      EDMA_TCD_CSR(29) = 0x08; //set DREQ
//    k70_set_handler(INT_DMA13, &int_dma13);
//    enable_irq(INT_DMA13 - 16);
    
    // SPI0 RX
    tcd1.saddr = (uint32)&SPI0_POPR;
    tcd1.daddr = (uint32)data_from_slave;
    tcd1.nbytes = 4;
    tcd1.tcdAttr = EDMA_TCD_ATTR_SSIZE_32BIT|EDMA_TCD_ATTR_DSIZE_32BIT ;
    tcd1.soff =0x00;
    tcd1.doff =0x04;
    tcd1.loopcount = 256;
    if (mux == 0)
      tcd1.channelno = 12;
    else
      tcd1.channelno = 28;
    
    dma_config(CONFIG_BASIC_XFR, &tcd1);
    
    // SPI1 TX	
    tcd2.saddr = (uint32)data_to_master;
    tcd2.daddr = (uint32)&SPI1_PUSHR;
    tcd2.nbytes = 4;
    tcd2.tcdAttr = EDMA_TCD_ATTR_SSIZE_32BIT|EDMA_TCD_ATTR_DSIZE_32BIT ;
    tcd2.soff =0x04;
    tcd2.doff =0x00;
    tcd2.loopcount = 256;
    if (mux == 0)
      tcd2.channelno = 15;
    else
      tcd2.channelno = 31;
    
    dma_config(CONFIG_BASIC_XFR, &tcd2);
    if (mux == 0)
      EDMA_TCD_CSR(15) = 0x08;
    else
      EDMA_TCD_CSR(31) = 0x08;
    
    // SPI1 RX
    tcd3.saddr = (uint32)&SPI1_POPR;
    tcd3.daddr = (uint32)data_from_master;
    tcd3.nbytes = 4;
    tcd3.tcdAttr = EDMA_TCD_ATTR_SSIZE_32BIT|EDMA_TCD_ATTR_DSIZE_32BIT ;
    tcd3.soff =0x00;
    tcd3.doff =0x04;
    tcd3.loopcount = 256;
    if (mux == 0)
      tcd3.channelno = 14;
    else
      tcd3.channelno = 30;
    
    dma_config(CONFIG_BASIC_XFR, &tcd3);    
    
    /* SPI configurations */
    dspi.br     = 0x00;
    dspi.cpha   = 0x00000000;
    dspi.cpol   = 0x00000000;
    
    // SPI0 config..
    SPI0_MCR = 0x80010C01;
    SPI0_CTAR0 = (0x78000000 +dspi.br) | dspi.cpha | dspi.cpol;
    SPI0_RSER = SPI_RSER_TFFF_RE|SPI_RSER_TFFF_DIRS;
    SPI0_RSER |= SPI_RSER_RFDF_RE|SPI_RSER_RFDF_DIRS;
    
    // SPI1 config...
    SPI1_MCR = 0x00010C01;
    SPI1_CTAR0 = 0x78000000 | dspi.cpha | dspi.cpol;
    SPI1_RSER = SPI_RSER_TFFF_RE|SPI_RSER_TFFF_DIRS;
    SPI1_RSER |= SPI_RSER_RFDF_RE|SPI_RSER_RFDF_DIRS;
    //Enable SPI1
    SPI1_MCR = 0x00010000;
  
    //Enable SPI0
    SPI0_MCR = 0x80010000;

    //Wait for eDMA fill TX FIFO completion
    dma_config(XFR_OVER_WAIT, &tcd); //SPI0_TX	
    dma_config(XFR_OVER_WAIT, &tcd2);//SPI1_TX	
    //Wait for eDMA completion
    dma_config(XFR_OVER_WAIT, &tcd3);//SPI1_RX
    dma_config(XFR_OVER_WAIT, &tcd1);//SPI0_RX
    
    while(SPI_SR_EOQF != (SPI0_SR & SPI_SR_EOQF))
    {}
    SPI0_MCR = 0x80010001;
    SPI1_MCR = 0x00010001;
    printf("eoqf bit is set\n");
    SPI0_SR = SPI0_SR | SPI_SR_TCF  | SPI_SR_EOQF ;
    printf( "SPI0_SR = 0x%08x\r\n",SPI0_SR);  
       
    //Print the datas    
    for(i=0; i<256 ; i++)
    {
        if((s16)data_to_slave[i] != data_from_master[i])
        {  
            printf("i = 0x%08x\r\n",i); 
            printf("data_to_slave = 0x%04x\r\n",(s16)data_to_slave[i]); 
            printf("data_from_master = 0x%04x\r\n",(s16)data_from_master[i]); 
            error_count++;
            printf("Transmit unsuccessful\n");
        }
    
        if((s16)data_to_master[i] != data_from_slave[i])
        {
            printf("i = 0x%08x\r\n",i); 
            printf("data_to_master = 0x%04x\r\n",(s16)data_to_master[i]); 
            printf("data_from_slave = 0x%04x\r\n",(s16)data_from_slave[i]); 
            error_count++;
            printf("Transmit unsuccessful\n");
        }
    
    }

    if (error_count)
    {
      printf("Number of Errors: %d\n",error_count);
    }
    else
    { 
      printf("No Errors Reported!\n");
    }
    
    /* disable */
    if (mux == 0)
    {
      DMA_MUX0_CHCONFIG15 = 0x00; 
      DMA_MUX0_CHCONFIG14 = 0x00; 
      DMA_MUX0_CHCONFIG13 = 0x00; 
      DMA_MUX0_CHCONFIG12 = 0x00; 
    }
    else
    {
      DMA_MUX1_CHCONFIG15 = 0x00; 
      DMA_MUX1_CHCONFIG14 = 0x00; 
      DMA_MUX1_CHCONFIG13 = 0x00; 
      DMA_MUX1_CHCONFIG12 = 0x00; 
       
    }  
    
    return(error_count);	
}
