/*
 * File:    pioneer3_edma.h
 * Purpose: Register and bit definitions
 */

#ifndef __PIONEER3_EDMA_H__
#define __PIONEER3_EDMA_H__

/*********************************************************************
*
* Enhanced DMA (EDMA)
*
*********************************************************************/
#define vuint32_t vuint32
#define vuint16_t vuint16
#define vuint8_t vuint8

/* peripheral base addresses */
#define EDMA_BASEADDR  0x40008000  // EDMA Starting Address

/* Register read/write macros */

#define EDMA_CR			(*(vuint32_t*)(EDMA_BASEADDR + 0x0000))
#define EDMA_ES            (*(vuint32_t*)(EDMA_BASEADDR + 0x0004))
#define EDMA_ERQL      	    (*(vuint32_t*)(EDMA_BASEADDR + 0x000C))
#define EDMA_EEIL          (*(vuint32_t*)(EDMA_BASEADDR + 0x0014))

#define EDMA_SERQ          (*(vuint8_t*)(EDMA_BASEADDR + 0x001B))
#define EDMA_CERQ          (*(vuint8_t*)(EDMA_BASEADDR + 0x001A))
#define EDMA_SEEI          (*(vuint8_t*)(EDMA_BASEADDR + 0x0019))
#define EDMA_CEEI          (*(vuint8_t*)(EDMA_BASEADDR + 0x0018))
#define EDMA_CINT          (*(vuint8_t*)(EDMA_BASEADDR + 0x001F))
#define EDMA_CERR          (*(vuint8_t*)(EDMA_BASEADDR + 0x001E))
#define EDMA_SSRT          (*(vuint8_t*)(EDMA_BASEADDR + 0x001D))
#define EDMA_CDNE          (*(vuint8_t*)(EDMA_BASEADDR + 0x001C))

#define EDMA_INTL          (*(vuint32_t*)(EDMA_BASEADDR + 0x0024))
#define EDMA_ERRL          (*(vuint32_t*)(EDMA_BASEADDR + 0x002C))
#define EDMA_HRSL          (*(vuint32_t*)(EDMA_BASEADDR + 0x0034))
#define EDMA_GPOR          (*(vuint32_t*)(EDMA_BASEADDR + 0x0038))

#define EDMA_DCHPRI0       (*(vuint8_t*)(EDMA_BASEADDR + 0x0103))
#define EDMA_DCHPRI1       (*(vuint8_t*)(EDMA_BASEADDR + 0x0102))
#define EDMA_DCHPRI2       (*(vuint8_t*)(EDMA_BASEADDR + 0x0101))
#define EDMA_DCHPRI3       (*(vuint8_t*)(EDMA_BASEADDR + 0x0100))
#define EDMA_DCHPRI4       (*(vuint8_t*)(EDMA_BASEADDR + 0x0107))
#define EDMA_DCHPRI5       (*(vuint8_t*)(EDMA_BASEADDR + 0x0106))
#define EDMA_DCHPRI6       (*(vuint8_t*)(EDMA_BASEADDR + 0x0105))
#define EDMA_DCHPRI7       (*(vuint8_t*)(EDMA_BASEADDR + 0x0104))
#define EDMA_DCHPRI8       (*(vuint8_t*)(EDMA_BASEADDR + 0x010B))
#define EDMA_DCHPRI9       (*(vuint8_t*)(EDMA_BASEADDR + 0x010A))
#define EDMA_DCHPRI10      (*(vuint8_t*)(EDMA_BASEADDR + 0x0109))
#define EDMA_DCHPRI11      (*(vuint8_t*)(EDMA_BASEADDR + 0x0108))
#define EDMA_DCHPRI12      (*(vuint8_t*)(EDMA_BASEADDR + 0x010F))
#define EDMA_DCHPRI13      (*(vuint8_t*)(EDMA_BASEADDR + 0x010E))
#define EDMA_DCHPRI14      (*(vuint8_t*)(EDMA_BASEADDR + 0x010D))
#define EDMA_DCHPRI15      (*(vuint8_t*)(EDMA_BASEADDR + 0x010C))
#define EDMA_DCHPRI(x) 	(*(vuint8_t*)((EDMA_BASEADDR + 0x0103) + ((x)*0x001)))

#define EDMA_TCD0_SADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x1000))
#define EDMA_TCD1_SADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x1020))
#define EDMA_TCD2_SADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x1040))
#define EDMA_TCD3_SADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x1060))
#define EDMA_TCD4_SADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x1080))
#define EDMA_TCD5_SADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x10A0))
#define EDMA_TCD6_SADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x10C0))
#define EDMA_TCD7_SADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x10E0))
#define EDMA_TCD8_SADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x1100))
#define EDMA_TCD9_SADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x1120))
#define EDMA_TCD10_SADDR   (*(vuint32_t*)(EDMA_BASEADDR + 0x1140))
#define EDMA_TCD11_SADDR   (*(vuint32_t*)(EDMA_BASEADDR + 0x1160))
#define EDMA_TCD12_SADDR   (*(vuint32_t*)(EDMA_BASEADDR + 0x1180))
#define EDMA_TCD13_SADDR   (*(vuint32_t*)(EDMA_BASEADDR + 0x11A0))
#define EDMA_TCD14_SADDR   (*(vuint32_t*)(EDMA_BASEADDR + 0x11C0))
#define EDMA_TCD15_SADDR   (*(vuint32_t*)(EDMA_BASEADDR + 0x11E0))
#define EDMA_TCD_SADDR(x)  (*(vuint32_t*)((EDMA_BASEADDR + 0x1000) + ((x)*0x020)))

#define EDMA_TCD0_SOFF     (*(vuint16_t*)(EDMA_BASEADDR + 0x1004))
#define EDMA_TCD1_SOFF     (*(vuint16_t*)(EDMA_BASEADDR + 0x1024))
#define EDMA_TCD2_SOFF     (*(vuint16_t*)(EDMA_BASEADDR + 0x1044))
#define EDMA_TCD3_SOFF     (*(vuint16_t*)(EDMA_BASEADDR + 0x1064))
#define EDMA_TCD4_SOFF     (*(vuint16_t*)(EDMA_BASEADDR + 0x1084))
#define EDMA_TCD5_SOFF     (*(vuint16_t*)(EDMA_BASEADDR + 0x10A4))
#define EDMA_TCD6_SOFF     (*(vuint16_t*)(EDMA_BASEADDR + 0x10C4))
#define EDMA_TCD7_SOFF     (*(vuint16_t*)(EDMA_BASEADDR + 0x10E4))
#define EDMA_TCD8_SOFF     (*(vuint16_t*)(EDMA_BASEADDR + 0x1104))
#define EDMA_TCD9_SOFF     (*(vuint16_t*)(EDMA_BASEADDR + 0x1124))
#define EDMA_TCD10_SOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x1144))
#define EDMA_TCD11_SOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x1164))
#define EDMA_TCD12_SOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x1184))
#define EDMA_TCD13_SOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x11A4))
#define EDMA_TCD14_SOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x11C4))
#define EDMA_TCD15_SOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x11E4))
#define EDMA_TCD_SOFF(x)   (*(vuint16_t*)((EDMA_BASEADDR + 0x1004) + ((x)*0x020)))

#define EDMA_TCD0_ATTR     (*(vuint16_t*)(EDMA_BASEADDR + 0x1006))
#define EDMA_TCD1_ATTR     (*(vuint16_t*)(EDMA_BASEADDR + 0x1026))
#define EDMA_TCD2_ATTR     (*(vuint16_t*)(EDMA_BASEADDR + 0x1046))
#define EDMA_TCD3_ATTR     (*(vuint16_t*)(EDMA_BASEADDR + 0x1066))
#define EDMA_TCD4_ATTR     (*(vuint16_t*)(EDMA_BASEADDR + 0x1086))
#define EDMA_TCD5_ATTR     (*(vuint16_t*)(EDMA_BASEADDR + 0x10A6))
#define EDMA_TCD6_ATTR     (*(vuint16_t*)(EDMA_BASEADDR + 0x10C6))
#define EDMA_TCD7_ATTR     (*(vuint16_t*)(EDMA_BASEADDR + 0x10E6))
#define EDMA_TCD8_ATTR     (*(vuint16_t*)(EDMA_BASEADDR + 0x1106))
#define EDMA_TCD9_ATTR     (*(vuint16_t*)(EDMA_BASEADDR + 0x1126))
#define EDMA_TCD10_ATTR    (*(vuint16_t*)(EDMA_BASEADDR + 0x1146))
#define EDMA_TCD11_ATTR    (*(vuint16_t*)(EDMA_BASEADDR + 0x1166))
#define EDMA_TCD12_ATTR    (*(vuint16_t*)(EDMA_BASEADDR + 0x1186))
#define EDMA_TCD13_ATTR    (*(vuint16_t*)(EDMA_BASEADDR + 0x11A6))
#define EDMA_TCD14_ATTR    (*(vuint16_t*)(EDMA_BASEADDR + 0x11C6))
#define EDMA_TCD15_ATTR    (*(vuint16_t*)(EDMA_BASEADDR + 0x11E6))
#define EDMA_TCD_ATTR(x)   (*(vuint16_t*)((EDMA_BASEADDR + 0x1006) + ((x)*0x020)))

#define EDMA_TCD0_NBYTES   (*(vuint32_t*)(EDMA_BASEADDR + 0x1008))
#define EDMA_TCD1_NBYTES   (*(vuint32_t*)(EDMA_BASEADDR + 0x1028))
#define EDMA_TCD2_NBYTES   (*(vuint32_t*)(EDMA_BASEADDR + 0x1048))
#define EDMA_TCD3_NBYTES   (*(vuint32_t*)(EDMA_BASEADDR + 0x1068))
#define EDMA_TCD4_NBYTES   (*(vuint32_t*)(EDMA_BASEADDR + 0x1088))
#define EDMA_TCD5_NBYTES   (*(vuint32_t*)(EDMA_BASEADDR + 0x10A8))
#define EDMA_TCD6_NBYTES   (*(vuint32_t*)(EDMA_BASEADDR + 0x10C8))
#define EDMA_TCD7_NBYTES   (*(vuint32_t*)(EDMA_BASEADDR + 0x10E8))
#define EDMA_TCD8_NBYTES   (*(vuint32_t*)(EDMA_BASEADDR + 0x1108))
#define EDMA_TCD9_NBYTES   (*(vuint32_t*)(EDMA_BASEADDR + 0x1128))
#define EDMA_TCD10_NBYTES  (*(vuint32_t*)(EDMA_BASEADDR + 0x1148))
#define EDMA_TCD11_NBYTES  (*(vuint32_t*)(EDMA_BASEADDR + 0x1168))
#define EDMA_TCD12_NBYTES  (*(vuint32_t*)(EDMA_BASEADDR + 0x1188))
#define EDMA_TCD13_NBYTES  (*(vuint32_t*)(EDMA_BASEADDR + 0x11A8))
#define EDMA_TCD14_NBYTES  (*(vuint32_t*)(EDMA_BASEADDR + 0x11C8))
#define EDMA_TCD15_NBYTES  (*(vuint32_t*)(EDMA_BASEADDR + 0x11E8))
#define EDMA_TCD_NBYTES(x) (*(vuint32_t*)((EDMA_BASEADDR + 0x1008) + ((x)*0x020)))

#define EDMA_TCD0_SLAST    (*(vuint32_t*)(EDMA_BASEADDR + 0x100C))
#define EDMA_TCD1_SLAST    (*(vuint32_t*)(EDMA_BASEADDR + 0x102C))
#define EDMA_TCD2_SLAST    (*(vuint32_t*)(EDMA_BASEADDR + 0x104C))
#define EDMA_TCD3_SLAST    (*(vuint32_t*)(EDMA_BASEADDR + 0x106C))
#define EDMA_TCD4_SLAST    (*(vuint32_t*)(EDMA_BASEADDR + 0x108C))
#define EDMA_TCD5_SLAST    (*(vuint32_t*)(EDMA_BASEADDR + 0x10AC))
#define EDMA_TCD6_SLAST    (*(vuint32_t*)(EDMA_BASEADDR + 0x10CC))
#define EDMA_TCD7_SLAST    (*(vuint32_t*)(EDMA_BASEADDR + 0x10EC))
#define EDMA_TCD8_SLAST    (*(vuint32_t*)(EDMA_BASEADDR + 0x110C))
#define EDMA_TCD9_SLAST    (*(vuint32_t*)(EDMA_BASEADDR + 0x112C))
#define EDMA_TCD10_SLAST   (*(vuint32_t*)(EDMA_BASEADDR + 0x114C))
#define EDMA_TCD11_SLAST   (*(vuint32_t*)(EDMA_BASEADDR + 0x116C))
#define EDMA_TCD12_SLAST   (*(vuint32_t*)(EDMA_BASEADDR + 0x118C))
#define EDMA_TCD13_SLAST   (*(vuint32_t*)(EDMA_BASEADDR + 0x11AC))
#define EDMA_TCD14_SLAST   (*(vuint32_t*)(EDMA_BASEADDR + 0x11CC))
#define EDMA_TCD15_SLAST   (*(vuint32_t*)(EDMA_BASEADDR + 0x11EC))
#define EDMA_TCD_SLAST(x)   (*(vuint32_t*)((EDMA_BASEADDR + 0x100C) + ((x)*0x020)))

#define EDMA_TCD0_DADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x1010))
#define EDMA_TCD1_DADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x1030))
#define EDMA_TCD2_DADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x1050))
#define EDMA_TCD3_DADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x1070))
#define EDMA_TCD4_DADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x1090))
#define EDMA_TCD5_DADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x10B0))
#define EDMA_TCD6_DADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x10D0))
#define EDMA_TCD7_DADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x10F0))
#define EDMA_TCD8_DADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x1110))
#define EDMA_TCD9_DADDR    (*(vuint32_t*)(EDMA_BASEADDR + 0x1130))
#define EDMA_TCD10_DADDR   (*(vuint32_t*)(EDMA_BASEADDR + 0x1150))
#define EDMA_TCD11_DADDR   (*(vuint32_t*)(EDMA_BASEADDR + 0x1170))
#define EDMA_TCD12_DADDR   (*(vuint32_t*)(EDMA_BASEADDR + 0x1190))
#define EDMA_TCD13_DADDR   (*(vuint32_t*)(EDMA_BASEADDR + 0x11B0))
#define EDMA_TCD14_DADDR   (*(vuint32_t*)(EDMA_BASEADDR + 0x11D0))
#define EDMA_TCD15_DADDR   (*(vuint32_t*)(EDMA_BASEADDR + 0x11F0))
#define EDMA_TCD_DADDR(x)  (*(vuint32_t*)((EDMA_BASEADDR + 0x1010) + ((x)*0x020)))

#define EDMA_TCD0_DOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x1014))
#define EDMA_TCD1_DOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x1034))
#define EDMA_TCD2_DOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x1054))
#define EDMA_TCD3_DOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x1074))
#define EDMA_TCD4_DOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x1094))
#define EDMA_TCD5_DOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x10B4))
#define EDMA_TCD6_DOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x10D4))
#define EDMA_TCD7_DOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x10F4))
#define EDMA_TCD8_DOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x1114))
#define EDMA_TCD9_DOFF    (*(vuint16_t*)(EDMA_BASEADDR + 0x1134))
#define EDMA_TCD10_DOFF   (*(vuint16_t*)(EDMA_BASEADDR + 0x1154))
#define EDMA_TCD11_DOFF   (*(vuint16_t*)(EDMA_BASEADDR + 0x1174))
#define EDMA_TCD12_DOFF   (*(vuint16_t*)(EDMA_BASEADDR + 0x1194))
#define EDMA_TCD13_DOFF   (*(vuint16_t*)(EDMA_BASEADDR + 0x11B4))
#define EDMA_TCD14_DOFF   (*(vuint16_t*)(EDMA_BASEADDR + 0x11D4))
#define EDMA_TCD15_DOFF   (*(vuint16_t*)(EDMA_BASEADDR + 0x11F4))
#define EDMA_TCD_DOFF(x)  (*(vuint16_t*)((EDMA_BASEADDR + 0x1014) + ((x)*0x020)))

#define EDMA_TCD0_CITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x1016))
#define EDMA_TCD1_CITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x1036))
#define EDMA_TCD2_CITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x1056))
#define EDMA_TCD3_CITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x1076))
#define EDMA_TCD4_CITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x1096))
#define EDMA_TCD5_CITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x10B6))
#define EDMA_TCD6_CITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x10D6))
#define EDMA_TCD7_CITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x10F6))
#define EDMA_TCD8_CITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x1116))
#define EDMA_TCD9_CITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x1136))
#define EDMA_TCD10_CITER   (*(vuint16_t*)(EDMA_BASEADDR + 0x1156))
#define EDMA_TCD11_CITER   (*(vuint16_t*)(EDMA_BASEADDR + 0x1176))
#define EDMA_TCD12_CITER   (*(vuint16_t*)(EDMA_BASEADDR + 0x1196))
#define EDMA_TCD13_CITER   (*(vuint16_t*)(EDMA_BASEADDR + 0x11B6))
#define EDMA_TCD14_CITER   (*(vuint16_t*)(EDMA_BASEADDR + 0x11D6))
#define EDMA_TCD15_CITER   (*(vuint16_t*)(EDMA_BASEADDR + 0x11F6))
#define EDMA_TCD_CITER(x)  (*(vuint16_t*)((EDMA_BASEADDR + 0x1016) + ((x)*0x020)))

#define EDMA_TCD0_CITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x1016))
#define EDMA_TCD1_CITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x1036))
#define EDMA_TCD2_CITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x1056))
#define EDMA_TCD3_CITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x1076))
#define EDMA_TCD4_CITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x1096))
#define EDMA_TCD5_CITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x10B6))
#define EDMA_TCD6_CITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x10D6))
#define EDMA_TCD7_CITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x10F6))
#define EDMA_TCD8_CITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x1116))
#define EDMA_TCD9_CITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x1136))
#define EDMA_TCD10_CITER_ELINK   (*(vuint16_t*)(EDMA_BASEADDR + 0x1156))
#define EDMA_TCD11_CITER_ELINK   (*(vuint16_t*)(EDMA_BASEADDR + 0x1176))
#define EDMA_TCD12_CITER_ELINK   (*(vuint16_t*)(EDMA_BASEADDR + 0x1196))
#define EDMA_TCD13_CITER_ELINK   (*(vuint16_t*)(EDMA_BASEADDR + 0x11B6))
#define EDMA_TCD14_CITER_ELINK   (*(vuint16_t*)(EDMA_BASEADDR + 0x11D6))
#define EDMA_TCD15_CITER_ELINK   (*(vuint16_t*)(EDMA_BASEADDR + 0x11F6))
#define EDMA_TCD_CITER_ELINK(x)  (*(vuint16_t*)((EDMA_BASEADDR + 0x1016) + ((x)*0x020)))

#define EDMA_TCD0_DLAST_SGA    (*(vuint32_t*)(EDMA_BASEADDR + 0x1018))
#define EDMA_TCD1_DLAST_SGA    (*(vuint32_t*)(EDMA_BASEADDR + 0x1038))
#define EDMA_TCD2_DLAST_SGA    (*(vuint32_t*)(EDMA_BASEADDR + 0x1058))
#define EDMA_TCD3_DLAST_SGA    (*(vuint32_t*)(EDMA_BASEADDR + 0x1078))
#define EDMA_TCD4_DLAST_SGA    (*(vuint32_t*)(EDMA_BASEADDR + 0x1098))
#define EDMA_TCD5_DLAST_SGA    (*(vuint32_t*)(EDMA_BASEADDR + 0x10B8))
#define EDMA_TCD6_DLAST_SGA    (*(vuint32_t*)(EDMA_BASEADDR + 0x10D8))
#define EDMA_TCD7_DLAST_SGA    (*(vuint32_t*)(EDMA_BASEADDR + 0x10F8))
#define EDMA_TCD8_DLAST_SGA    (*(vuint32_t*)(EDMA_BASEADDR + 0x1118))
#define EDMA_TCD9_DLAST_SGA    (*(vuint32_t*)(EDMA_BASEADDR + 0x1138))
#define EDMA_TCD10_DLAST_SGA   (*(vuint32_t*)(EDMA_BASEADDR + 0x1158))
#define EDMA_TCD11_DLAST_SGA   (*(vuint32_t*)(EDMA_BASEADDR + 0x1178))
#define EDMA_TCD12_DLAST_SGA   (*(vuint32_t*)(EDMA_BASEADDR + 0x1198))
#define EDMA_TCD13_DLAST_SGA   (*(vuint32_t*)(EDMA_BASEADDR + 0x11B8))
#define EDMA_TCD14_DLAST_SGA   (*(vuint32_t*)(EDMA_BASEADDR + 0x11D8))
#define EDMA_TCD15_DLAST_SGA   (*(vuint32_t*)(EDMA_BASEADDR + 0x11F8))
#define EDMA_TCD_DLAST_SGA(x)  (*(vuint32_t*)((EDMA_BASEADDR + 0x1018) + ((x)*0x020)))

#define EDMA_TCD0_CSR    (*(vuint16_t*)(EDMA_BASEADDR + 0x101C))
#define EDMA_TCD1_CSR    (*(vuint16_t*)(EDMA_BASEADDR + 0x103C))
#define EDMA_TCD2_CSR    (*(vuint16_t*)(EDMA_BASEADDR + 0x105C))
#define EDMA_TCD3_CSR    (*(vuint16_t*)(EDMA_BASEADDR + 0x107C))
#define EDMA_TCD4_CSR    (*(vuint16_t*)(EDMA_BASEADDR + 0x109C))
#define EDMA_TCD5_CSR    (*(vuint16_t*)(EDMA_BASEADDR + 0x10BC))
#define EDMA_TCD6_CSR    (*(vuint16_t*)(EDMA_BASEADDR + 0x10DC))
#define EDMA_TCD7_CSR    (*(vuint16_t*)(EDMA_BASEADDR + 0x10FC))
#define EDMA_TCD8_CSR    (*(vuint16_t*)(EDMA_BASEADDR + 0x111C))
#define EDMA_TCD9_CSR    (*(vuint16_t*)(EDMA_BASEADDR + 0x113C))
#define EDMA_TCD10_CSR   (*(vuint16_t*)(EDMA_BASEADDR + 0x115C))
#define EDMA_TCD11_CSR   (*(vuint16_t*)(EDMA_BASEADDR + 0x117C))
#define EDMA_TCD12_CSR   (*(vuint16_t*)(EDMA_BASEADDR + 0x119C))
#define EDMA_TCD13_CSR   (*(vuint16_t*)(EDMA_BASEADDR + 0x11BC))
#define EDMA_TCD14_CSR   (*(vuint16_t*)(EDMA_BASEADDR + 0x11DC))
#define EDMA_TCD15_CSR   (*(vuint16_t*)(EDMA_BASEADDR + 0x11FC))
#define EDMA_TCD_CSR(x)  (*(vuint16_t*)((EDMA_BASEADDR + 0x101C) + ((x)*0x020)))

#define EDMA_TCD0_BITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x101E))
#define EDMA_TCD1_BITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x103E))
#define EDMA_TCD2_BITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x105E))
#define EDMA_TCD3_BITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x107E))
#define EDMA_TCD4_BITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x109E))
#define EDMA_TCD5_BITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x10BE))
#define EDMA_TCD6_BITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x10DE))
#define EDMA_TCD7_BITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x10FE))
#define EDMA_TCD8_BITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x111E))
#define EDMA_TCD9_BITER    (*(vuint16_t*)(EDMA_BASEADDR + 0x113E))
#define EDMA_TCD10_BITER   (*(vuint16_t*)(EDMA_BASEADDR + 0x115E))
#define EDMA_TCD11_BITER   (*(vuint16_t*)(EDMA_BASEADDR + 0x117E))
#define EDMA_TCD12_BITER   (*(vuint16_t*)(EDMA_BASEADDR + 0x119E))
#define EDMA_TCD13_BITER   (*(vuint16_t*)(EDMA_BASEADDR + 0x11BE))
#define EDMA_TCD14_BITER   (*(vuint16_t*)(EDMA_BASEADDR + 0x11DE))
#define EDMA_TCD15_BITER   (*(vuint16_t*)(EDMA_BASEADDR + 0x11FE))
#define EDMA_TCD_BITER(x)  (*(vuint16_t*)((EDMA_BASEADDR + 0x101E) + ((x)*0x020)))

#define EDMA_TCD0_BITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x101E))
#define EDMA_TCD1_BITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x103E))
#define EDMA_TCD2_BITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x105E))
#define EDMA_TCD3_BITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x107E))
#define EDMA_TCD4_BITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x109E))
#define EDMA_TCD5_BITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x10BE))
#define EDMA_TCD6_BITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x10DE))
#define EDMA_TCD7_BITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x10FE))
#define EDMA_TCD8_BITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x111E))
#define EDMA_TCD9_BITER_ELINK    (*(vuint16_t*)(EDMA_BASEADDR + 0x113E))
#define EDMA_TCD10_BITER_ELINK   (*(vuint16_t*)(EDMA_BASEADDR + 0x115E))
#define EDMA_TCD11_BITER_ELINK   (*(vuint16_t*)(EDMA_BASEADDR + 0x117E))
#define EDMA_TCD12_BITER_ELINK   (*(vuint16_t*)(EDMA_BASEADDR + 0x119E))
#define EDMA_TCD13_BITER_ELINK   (*(vuint16_t*)(EDMA_BASEADDR + 0x11BE))
#define EDMA_TCD14_BITER_ELINK   (*(vuint16_t*)(EDMA_BASEADDR + 0x11DE))
#define EDMA_TCD15_BITER_ELINK   (*(vuint16_t*)(EDMA_BASEADDR + 0x11FE))
#define EDMA_TCD_BITER_ELINK(x)  (*(vuint16_t*)((EDMA_BASEADDR + 0x101E) + ((x)*0x020)))

/***********************************************************************/
/* Bit definitions and macros for EDMA_CR */
#define EDMA_CR_EBW                      (0x00000001)
#define EDMA_CR_EDBG                     (0x00000002)
#define EDMA_CR_ERCA                     (0x00000004)
#define EDMA_CR_ERGA                     (0x00000008)
//#define EDMA_CR_GRP0PR(x)                (((x)&0x00000003)<<8)
//#define EDMA_CR_GRP1PRI(x)               (((x)&0x00000003)<<10)
//#define EDMA_CR_GRP2PRI(x)               (((x)&0x00000003)<<12)
//#define EDMA_CR_GRP3PRI(x)               (((x)&0x00000003)<<14)
#define EDMA_CR_ECX                      (0x00010000)
#define EDMA_CR_CX                       (0x00020000)

/* Bit definitions and macros for EDMA_ES */
#define EDMA_ES_DBE                      (0x00000001)
#define EDMA_ES_SBE                      (0x00000002)
#define EDMA_ES_SGE                      (0x00000004)
#define EDMA_ES_NCE                      (0x00000008)
#define EDMA_ES_DOE                      (0x00000010)
#define EDMA_ES_DAE                      (0x00000020)
#define EDMA_ES_SOE                      (0x00000040)
#define EDMA_ES_SAE                      (0x00000080)
#define EDMA_ES_ERRCHN(x)                (((x)&0x0000003F)<<8)
#define EDMA_ES_CPE                      (0x00004000)
//#define EDMA_ES_GPE                      (0x00008000)
#define EDMA_ES_ECX                      (0x00010000)
#define EDMA_ES_VLD                      (0x80000000)

/* Bit definitions and macros for EDMA_ERQH */
/*
#define EDMA_ERQH_ERQH32                 (0x00000001)
#define EDMA_ERQH_ERQH33                 (0x00000002)
#define EDMA_ERQH_ERQH34                 (0x00000004)
#define EDMA_ERQH_ERQH35                 (0x00000008)
#define EDMA_ERQH_ERQH36                 (0x00000010)
#define EDMA_ERQH_ERQH37                 (0x00000020)
#define EDMA_ERQH_ERQH38                 (0x00000040)
#define EDMA_ERQH_ERQH39                 (0x00000080)
#define EDMA_ERQH_ERQH40                 (0x00000100)
#define EDMA_ERQH_ERQH41                 (0x00000200)
#define EDMA_ERQH_ERQH42                 (0x00000400)
#define EDMA_ERQH_ERQH43                 (0x00000800)
#define EDMA_ERQH_ERQH44                 (0x00001000)
#define EDMA_ERQH_ERQH45                 (0x00002000)
#define EDMA_ERQH_ERQH46                 (0x00004000)
#define EDMA_ERQH_ERQH47                 (0x00008000)
#define EDMA_ERQH_ERQH48                 (0x00010000)
#define EDMA_ERQH_ERQH49                 (0x00020000)
#define EDMA_ERQH_ERQH50                 (0x00040000)
#define EDMA_ERQH_ERQH51                 (0x00080000)
#define EDMA_ERQH_ERQH52                 (0x00100000)
#define EDMA_ERQH_ERQH53                 (0x00200000)
#define EDMA_ERQH_ERQH54                 (0x00400000)
#define EDMA_ERQH_ERQH55                 (0x00800000)
#define EDMA_ERQH_ERQH56                 (0x01000000)
#define EDMA_ERQH_ERQH57                 (0x02000000)
#define EDMA_ERQH_ERQH58                 (0x04000000)
#define EDMA_ERQH_ERQH59                 (0x08000000)
#define EDMA_ERQH_ERQH60                 (0x10000000)
#define EDMA_ERQH_ERQH61                 (0x20000000)
#define EDMA_ERQH_ERQH62                 (0x40000000)
#define EDMA_ERQH_ERQH63                 (0x80000000)
*/
/* Bit definitions and macros for EDMA_ERQL */
#define EDMA_ERQL_ERQL0                  (0x00000001)
#define EDMA_ERQL_ERQL1                  (0x00000002)
#define EDMA_ERQL_ERQL2                  (0x00000004)
#define EDMA_ERQL_ERQL3                  (0x00000008)
#define EDMA_ERQL_ERQL4                  (0x00000010)
#define EDMA_ERQL_ERQL5                  (0x00000020)
#define EDMA_ERQL_ERQL6                  (0x00000040)
#define EDMA_ERQL_ERQL7                  (0x00000080)
#define EDMA_ERQL_ERQL8                  (0x00000100)
#define EDMA_ERQL_ERQL9                  (0x00000200)
#define EDMA_ERQL_ERQL10                 (0x00000400)
#define EDMA_ERQL_ERQL11                 (0x00000800)
#define EDMA_ERQL_ERQL12                 (0x00001000)
#define EDMA_ERQL_ERQL13                 (0x00002000)
#define EDMA_ERQL_ERQL14                 (0x00004000)
#define EDMA_ERQL_ERQL15                 (0x00008000)
#define EDMA_ERQL_ERQL16                 (0x00010000)
#define EDMA_ERQL_ERQL17                 (0x00020000)
#define EDMA_ERQL_ERQL18                 (0x00040000)
#define EDMA_ERQL_ERQL19                 (0x00080000)
#define EDMA_ERQL_ERQL20                 (0x00100000)
#define EDMA_ERQL_ERQL21                 (0x00200000)
#define EDMA_ERQL_ERQL22                 (0x00400000)
#define EDMA_ERQL_ERQL23                 (0x00800000)
#define EDMA_ERQL_ERQL24                 (0x01000000)
#define EDMA_ERQL_ERQL25                 (0x02000000)
#define EDMA_ERQL_ERQL26                 (0x04000000)
#define EDMA_ERQL_ERQL27                 (0x08000000)
#define EDMA_ERQL_ERQL28                 (0x10000000)
#define EDMA_ERQL_ERQL29                 (0x20000000)
#define EDMA_ERQL_ERQL30                 (0x40000000)
#define EDMA_ERQL_ERQL31                 (0x80000000)

/* Bit definitions and macros for EDMA_EEIH */
/*
#define EDMA_EEIH_EEIH32                 (0x00000001)
#define EDMA_EEIH_EEIH33                 (0x00000002)
#define EDMA_EEIH_EEIH34                 (0x00000004)
#define EDMA_EEIH_EEIH35                 (0x00000008)
#define EDMA_EEIH_EEIH36                 (0x00000010)
#define EDMA_EEIH_EEIH37                 (0x00000020)
#define EDMA_EEIH_EEIH38                 (0x00000040)
#define EDMA_EEIH_EEIH39                 (0x00000080)
#define EDMA_EEIH_EEIH40                 (0x00000100)
#define EDMA_EEIH_EEIH41                 (0x00000200)
#define EDMA_EEIH_EEIH42                 (0x00000400)
#define EDMA_EEIH_EEIH43                 (0x00000800)
#define EDMA_EEIH_EEIH44                 (0x00001000)
#define EDMA_EEIH_EEIH45                 (0x00002000)
#define EDMA_EEIH_EEIH46                 (0x00004000)
#define EDMA_EEIH_EEIH47                 (0x00008000)
#define EDMA_EEIH_EEIH48                 (0x00010000)
#define EDMA_EEIH_EEIH49                 (0x00020000)
#define EDMA_EEIH_EEIH50                 (0x00040000)
#define EDMA_EEIH_EEIH51                 (0x00080000)
#define EDMA_EEIH_EEIH52                 (0x00100000)
#define EDMA_EEIH_EEIH53                 (0x00200000)
#define EDMA_EEIH_EEIH54                 (0x00400000)
#define EDMA_EEIH_EEIH55                 (0x00800000)
#define EDMA_EEIH_EEIH56                 (0x01000000)
#define EDMA_EEIH_EEIH57                 (0x02000000)
#define EDMA_EEIH_EEIH58                 (0x04000000)
#define EDMA_EEIH_EEIH59                 (0x08000000)
#define EDMA_EEIH_EEIH60                 (0x10000000)
#define EDMA_EEIH_EEIH61                 (0x20000000)
#define EDMA_EEIH_EEIH62                 (0x40000000)
#define EDMA_EEIH_EEIH63                 (0x80000000)
*/
/* Bit definitions and macros for EDMA_EEIL */
#define EDMA_EEIL_EEIL0                  (0x00000001)
#define EDMA_EEIL_EEIL1                  (0x00000002)
#define EDMA_EEIL_EEIL2                  (0x00000004)
#define EDMA_EEIL_EEIL3                  (0x00000008)
#define EDMA_EEIL_EEIL4                  (0x00000010)
#define EDMA_EEIL_EEIL5                  (0x00000020)
#define EDMA_EEIL_EEIL6                  (0x00000040)
#define EDMA_EEIL_EEIL7                  (0x00000080)
#define EDMA_EEIL_EEIL8                  (0x00000100)
#define EDMA_EEIL_EEIL9                  (0x00000200)
#define EDMA_EEIL_EEIL10                 (0x00000400)
#define EDMA_EEIL_EEIL11                 (0x00000800)
#define EDMA_EEIL_EEIL12                 (0x00001000)
#define EDMA_EEIL_EEIL13                 (0x00002000)
#define EDMA_EEIL_EEIL14                 (0x00004000)
#define EDMA_EEIL_EEIL15                 (0x00008000)
#define EDMA_EEIL_EEIL16                 (0x00010000)
#define EDMA_EEIL_EEIL17                 (0x00020000)
#define EDMA_EEIL_EEIL18                 (0x00040000)
#define EDMA_EEIL_EEIL19                 (0x00080000)
#define EDMA_EEIL_EEIL20                 (0x00100000)
#define EDMA_EEIL_EEIL21                 (0x00200000)
#define EDMA_EEIL_EEIL22                 (0x00400000)
#define EDMA_EEIL_EEIL23                 (0x00800000)
#define EDMA_EEIL_EEIL24                 (0x01000000)
#define EDMA_EEIL_EEIL25                 (0x02000000)
#define EDMA_EEIL_EEIL26                 (0x04000000)
#define EDMA_EEIL_EEIL27                 (0x08000000)
#define EDMA_EEIL_EEIL28                 (0x10000000)
#define EDMA_EEIL_EEIL29                 (0x20000000)
#define EDMA_EEIL_EEIL30                 (0x40000000)
#define EDMA_EEIL_EEIL31                 (0x80000000)

/* Bit definitions and macros for EDMA_SERQ */
#define EDMA_SERQ_SERQ(x)                (((x)&0x7F)<<0)
#define EDMA_SERQ_SAER                   (0x80)

/* Bit definitions and macros for EDMA_CERQ */
#define EDMA_CERQ_CERQ(x)                (((x)&0x7F)<<0)
#define EDMA_CERQ_CAER                   (0x80)

/* Bit definitions and macros for EDMA_SEEI */
#define EDMA_SEEI_SEEI(x)                (((x)&0x7F)<<0)
#define EDMA_SEEI_SAEE                   (0x80)

/* Bit definitions and macros for EDMA_CEEI */
#define EDMA_CEEI_CEEI(x)                (((x)&0x7F)<<0)
#define EDMA_CEEI_CAEE                   (0x80)

/* Bit definitions and macros for EDMA_CINT */
#define EDMA_CINT_CINT(x)                (((x)&0x7F)<<0)
#define EDMA_CINT_CAIR                   (0x80)

/* Bit definitions and macros for EDMA_CERR */
#define EDMA_CERR_CERR(x)                (((x)&0x7F)<<0)
#define EDMA_CERR_CAER                   (0x80)

/* Bit definitions and macros for EDMA_SSRT */
#define EDMA_SSRT_SSRT(x)                (((x)&0x7F)<<0)
#define EDMA_SSRT_SAST                   (0x80)

/* Bit definitions and macros for EDMA_CDNE */
#define EDMA_CDNE_CDNE(x)                (((x)&0x7F)<<0)
#define EDMA_CDNE_CADN                   (0x80)

/* Bit definitions and macros for EDMA_INTH */
/*
#define EDMA_INTH_INTH32                 (0x00000001)
#define EDMA_INTH_INTH33                 (0x00000002)
#define EDMA_INTH_INTH34                 (0x00000004)
#define EDMA_INTH_INTH35                 (0x00000008)
#define EDMA_INTH_INTH36                 (0x00000010)
#define EDMA_INTH_INTH37                 (0x00000020)
#define EDMA_INTH_INTH38                 (0x00000040)
#define EDMA_INTH_INTH39                 (0x00000080)
#define EDMA_INTH_INTH40                 (0x00000100)
#define EDMA_INTH_INTH41                 (0x00000200)
#define EDMA_INTH_INTH42                 (0x00000400)
#define EDMA_INTH_INTH43                 (0x00000800)
#define EDMA_INTH_INTH44                 (0x00001000)
#define EDMA_INTH_INTH45                 (0x00002000)
#define EDMA_INTH_INTH46                 (0x00004000)
#define EDMA_INTH_INTH47                 (0x00008000)
#define EDMA_INTH_INTH48                 (0x00010000)
#define EDMA_INTH_INTH49                 (0x00020000)
#define EDMA_INTH_INTH50                 (0x00040000)
#define EDMA_INTH_INTH51                 (0x00080000)
#define EDMA_INTH_INTH52                 (0x00100000)
#define EDMA_INTH_INTH53                 (0x00200000)
#define EDMA_INTH_INTH54                 (0x00400000)
#define EDMA_INTH_INTH55                 (0x00800000)
#define EDMA_INTH_INTH56                 (0x01000000)
#define EDMA_INTH_INTH57                 (0x02000000)
#define EDMA_INTH_INTH58                 (0x04000000)
#define EDMA_INTH_INTH59                 (0x08000000)
#define EDMA_INTH_INTH60                 (0x10000000)
#define EDMA_INTH_INTH61                 (0x20000000)
#define EDMA_INTH_INTH62                 (0x40000000)
#define EDMA_INTH_INTH63                 (0x80000000)
*/
/* Bit definitions and macros for EDMA_INTL */
#define EDMA_INTL_INTL0                  (0x00000001)
#define EDMA_INTL_INTL1                  (0x00000002)
#define EDMA_INTL_INTL2                  (0x00000004)
#define EDMA_INTL_INTL3                  (0x00000008)
#define EDMA_INTL_INTL4                  (0x00000010)
#define EDMA_INTL_INTL5                  (0x00000020)
#define EDMA_INTL_INTL6                  (0x00000040)
#define EDMA_INTL_INTL7                  (0x00000080)
#define EDMA_INTL_INTL8                  (0x00000100)
#define EDMA_INTL_INTL9                  (0x00000200)
#define EDMA_INTL_INTL10                 (0x00000400)
#define EDMA_INTL_INTL11                 (0x00000800)
#define EDMA_INTL_INTL12                 (0x00001000)
#define EDMA_INTL_INTL13                 (0x00002000)
#define EDMA_INTL_INTL14                 (0x00004000)
#define EDMA_INTL_INTL15                 (0x00008000)
#define EDMA_INTL_INTL16                 (0x00010000)
#define EDMA_INTL_INTL17                 (0x00020000)
#define EDMA_INTL_INTL18                 (0x00040000)
#define EDMA_INTL_INTL19                 (0x00080000)
#define EDMA_INTL_INTL20                 (0x00100000)
#define EDMA_INTL_INTL21                 (0x00200000)
#define EDMA_INTL_INTL22                 (0x00400000)
#define EDMA_INTL_INTL23                 (0x00800000)
#define EDMA_INTL_INTL24                 (0x01000000)
#define EDMA_INTL_INTL25                 (0x02000000)
#define EDMA_INTL_INTL26                 (0x04000000)
#define EDMA_INTL_INTL27                 (0x08000000)
#define EDMA_INTL_INTL28                 (0x10000000)
#define EDMA_INTL_INTL29                 (0x20000000)
#define EDMA_INTL_INTL30                 (0x40000000)
#define EDMA_INTL_INTL31                 (0x80000000)

/* Bit definitions and macros for EDMA_ERRH */
/*
#define EDMA_ERRH_ERRH32                 (0x00000001)
#define EDMA_ERRH_ERRH33                 (0x00000002)
#define EDMA_ERRH_ERRH34                 (0x00000004)
#define EDMA_ERRH_ERRH35                 (0x00000008)
#define EDMA_ERRH_ERRH36                 (0x00000010)
#define EDMA_ERRH_ERRH37                 (0x00000020)
#define EDMA_ERRH_ERRH38                 (0x00000040)
#define EDMA_ERRH_ERRH39                 (0x00000080)
#define EDMA_ERRH_ERRH40                 (0x00000100)
#define EDMA_ERRH_ERRH41                 (0x00000200)
#define EDMA_ERRH_ERRH42                 (0x00000400)
#define EDMA_ERRH_ERRH43                 (0x00000800)
#define EDMA_ERRH_ERRH44                 (0x00001000)
#define EDMA_ERRH_ERRH45                 (0x00002000)
#define EDMA_ERRH_ERRH46                 (0x00004000)
#define EDMA_ERRH_ERRH47                 (0x00008000)
#define EDMA_ERRH_ERRH48                 (0x00010000)
#define EDMA_ERRH_ERRH49                 (0x00020000)
#define EDMA_ERRH_ERRH50                 (0x00040000)
#define EDMA_ERRH_ERRH51                 (0x00080000)
#define EDMA_ERRH_ERRH52                 (0x00100000)
#define EDMA_ERRH_ERRH53                 (0x00200000)
#define EDMA_ERRH_ERRH54                 (0x00400000)
#define EDMA_ERRH_ERRH55                 (0x00800000)
#define EDMA_ERRH_ERRH56                 (0x01000000)
#define EDMA_ERRH_ERRH57                 (0x02000000)
#define EDMA_ERRH_ERRH58                 (0x04000000)
#define EDMA_ERRH_ERRH59                 (0x08000000)
#define EDMA_ERRH_ERRH60                 (0x10000000)
#define EDMA_ERRH_ERRH61                 (0x20000000)
#define EDMA_ERRH_ERRH62                 (0x40000000)
#define EDMA_ERRH_ERRH63                 (0x80000000)
*/
/* Bit definitions and macros for EDMA_ERRL */
#define EDMA_ERRL_ERRL0                  (0x00000001)
#define EDMA_ERRL_ERRL1                  (0x00000002)
#define EDMA_ERRL_ERRL2                  (0x00000004)
#define EDMA_ERRL_ERRL3                  (0x00000008)
#define EDMA_ERRL_ERRL4                  (0x00000010)
#define EDMA_ERRL_ERRL5                  (0x00000020)
#define EDMA_ERRL_ERRL6                  (0x00000040)
#define EDMA_ERRL_ERRL7                  (0x00000080)
#define EDMA_ERRL_ERRL8                  (0x00000100)
#define EDMA_ERRL_ERRL9                  (0x00000200)
#define EDMA_ERRL_ERRL10                 (0x00000400)
#define EDMA_ERRL_ERRL11                 (0x00000800)
#define EDMA_ERRL_ERRL12                 (0x00001000)
#define EDMA_ERRL_ERRL13                 (0x00002000)
#define EDMA_ERRL_ERRL14                 (0x00004000)
#define EDMA_ERRL_ERRL15                 (0x00008000)
#define EDMA_ERRL_ERRL16                 (0x00010000)
#define EDMA_ERRL_ERRL17                 (0x00020000)
#define EDMA_ERRL_ERRL18                 (0x00040000)
#define EDMA_ERRL_ERRL19                 (0x00080000)
#define EDMA_ERRL_ERRL20                 (0x00100000)
#define EDMA_ERRL_ERRL21                 (0x00200000)
#define EDMA_ERRL_ERRL22                 (0x00400000)
#define EDMA_ERRL_ERRL23                 (0x00800000)
#define EDMA_ERRL_ERRL24                 (0x01000000)
#define EDMA_ERRL_ERRL25                 (0x02000000)
#define EDMA_ERRL_ERRL26                 (0x04000000)
#define EDMA_ERRL_ERRL27                 (0x08000000)
#define EDMA_ERRL_ERRL28                 (0x10000000)
#define EDMA_ERRL_ERRL29                 (0x20000000)
#define EDMA_ERRL_ERRL30                 (0x40000000)
#define EDMA_ERRL_ERRL31                 (0x80000000)

/* Bit definitions and macros for EDMA_HRSH */
/*
#define EDMA_HRSH_HRSH32                 (0x00000001)
#define EDMA_HRSH_HRSH33                 (0x00000002)
#define EDMA_HRSH_HRSH34                 (0x00000004)
#define EDMA_HRSH_HRSH35                 (0x00000008)
#define EDMA_HRSH_HRSH36                 (0x00000010)
#define EDMA_HRSH_HRSH37                 (0x00000020)
#define EDMA_HRSH_HRSH38                 (0x00000040)
#define EDMA_HRSH_HRSH39                 (0x00000080)
#define EDMA_HRSH_HRSH40                 (0x00000100)
#define EDMA_HRSH_HRSH41                 (0x00000200)
#define EDMA_HRSH_HRSH42                 (0x00000400)
#define EDMA_HRSH_HRSH43                 (0x00000800)
#define EDMA_HRSH_HRSH44                 (0x00001000)
#define EDMA_HRSH_HRSH45                 (0x00002000)
#define EDMA_HRSH_HRSH46                 (0x00004000)
#define EDMA_HRSH_HRSH47                 (0x00008000)
#define EDMA_HRSH_HRSH48                 (0x00010000)
#define EDMA_HRSH_HRSH49                 (0x00020000)
#define EDMA_HRSH_HRSH50                 (0x00040000)
#define EDMA_HRSH_HRSH51                 (0x00080000)
#define EDMA_HRSH_HRSH52                 (0x00100000)
#define EDMA_HRSH_HRSH53                 (0x00200000)
#define EDMA_HRSH_HRSH54                 (0x00400000)
#define EDMA_HRSH_HRSH55                 (0x00800000)
#define EDMA_HRSH_HRSH56                 (0x01000000)
#define EDMA_HRSH_HRSH57                 (0x02000000)
#define EDMA_HRSH_HRSH58                 (0x04000000)
#define EDMA_HRSH_HRSH59                 (0x08000000)
#define EDMA_HRSH_HRSH60                 (0x10000000)
#define EDMA_HRSH_HRSH61                 (0x20000000)
#define EDMA_HRSH_HRSH62                 (0x40000000)
#define EDMA_HRSH_HRSH63                 (0x80000000)
*/
/* Bit definitions and macros for EDMA_HRSL */
#define EDMA_HRSL_HRSL0                  (0x00000001)
#define EDMA_HRSL_HRSL1                  (0x00000002)
#define EDMA_HRSL_HRSL2                  (0x00000004)
#define EDMA_HRSL_HRSL3                  (0x00000008)
#define EDMA_HRSL_HRSL4                  (0x00000010)
#define EDMA_HRSL_HRSL5                  (0x00000020)
#define EDMA_HRSL_HRSL6                  (0x00000040)
#define EDMA_HRSL_HRSL7                  (0x00000080)
#define EDMA_HRSL_HRSL8                  (0x00000100)
#define EDMA_HRSL_HRSL9                  (0x00000200)
#define EDMA_HRSL_HRSL10                 (0x00000400)
#define EDMA_HRSL_HRSL11                 (0x00000800)
#define EDMA_HRSL_HRSL12                 (0x00001000)
#define EDMA_HRSL_HRSL13                 (0x00002000)
#define EDMA_HRSL_HRSL14                 (0x00004000)
#define EDMA_HRSL_HRSL15                 (0x00008000)
#define EDMA_HRSL_HRSL16                 (0x00010000)
#define EDMA_HRSL_HRSL17                 (0x00020000)
#define EDMA_HRSL_HRSL18                 (0x00040000)
#define EDMA_HRSL_HRSL19                 (0x00080000)
#define EDMA_HRSL_HRSL20                 (0x00100000)
#define EDMA_HRSL_HRSL21                 (0x00200000)
#define EDMA_HRSL_HRSL22                 (0x00400000)
#define EDMA_HRSL_HRSL23                 (0x00800000)
#define EDMA_HRSL_HRSL24                 (0x01000000)
#define EDMA_HRSL_HRSL25                 (0x02000000)
#define EDMA_HRSL_HRSL26                 (0x04000000)
#define EDMA_HRSL_HRSL27                 (0x08000000)
#define EDMA_HRSL_HRSL28                 (0x10000000)
#define EDMA_HRSL_HRSL29                 (0x20000000)
#define EDMA_HRSL_HRSL30                 (0x40000000)
#define EDMA_HRSL_HRSL31                 (0x80000000)

/* Bit definitions and macros for EDMA_GPOR */
//#define MCF_EDMA_GPOR_GPOR(x)                (((x)&0xFFFFFFFF)<<0)

/* Bit definitions and macros for EDMA_DCHPRI */
#define EDMA_DCHPRI_CHPRI(x)             (((x)&0x0F)<<0)
//#define EDMA_DCHPRI_GRPPR(x)             (((x)&0x03)<<4)
#define EDMA_DCHPRI_DPA                  (0x40)
#define EDMA_DCHPRI_ECP                  (0x80)

/* Bit definitions and macros for EDMA_TCD_SADDR */
#define EDMA_TCD_SADDR_SADDR(x)          (((x)&0xFFFFFFFF)<<0)

/* Bit definitions and macros for EDMA_TCD_ATTR */
#define EDMA_TCD_ATTR_DSIZE(x)           (((x)&0x0007)<<0)
#define EDMA_TCD_ATTR_DMOD(x)            (((x)&0x001F)<<3)
#define EDMA_TCD_ATTR_SSIZE(x)           (((x)&0x0007)<<8)
#define EDMA_TCD_ATTR_SMOD(x)            (((x)&0x001F)<<11)
#define EDMA_TCD_ATTR_SSIZE_8BIT         (0x0000)
#define EDMA_TCD_ATTR_SSIZE_16BIT        (0x0100)
#define EDMA_TCD_ATTR_SSIZE_32BIT        (0x0200)
#define EDMA_TCD_ATTR_SSIZE_64BIT        (0x0300)
#define EDMA_TCD_ATTR_SSIZE_16BYTE       (0x0400)
#define EDMA_TCD_ATTR_SSIZE_32BYTE       (0x0500)
#define EDMA_TCD_ATTR_DSIZE_8BIT         (0x0000)
#define EDMA_TCD_ATTR_DSIZE_16BIT        (0x0001)
#define EDMA_TCD_ATTR_DSIZE_32BIT        (0x0002)
#define EDMA_TCD_ATTR_DSIZE_64BIT        (0x0003)
#define EDMA_TCD_ATTR_DSIZE_16BYTE       (0x0004)
#define EDMA_TCD_ATTR_DSIZE_32BYTE       (0x0005)

/* Bit definitions and macros for EDMA_TCD_SOFF */
#define EDMA_TCD_SOFF_SOFF(x)            (((x)&0xFFFF)<<0)

/* Bit definitions and macros for EDMA_TCD_NBYTES */
#define EDMA_TCD_NBYTES_NBYTES(x)        (((x)&0x000000FF)<<0)
#define EDMA_TCD_NBYTES_MLOF(x)          (((x)&0x003FFFFF)<<8)
#define EDMA_TCD_NBYTES_DMLOE            (0x40000000)
#define EDMA_TCD_NBYTES_SMLOE            (0x80000000)

/* Bit definitions and macros for EDMA_TCD_SLAST */
#define  EDMA_TCD_SLAST_SLAST(x)          (((x)&0xFFFFFFFF)<<0)

/* Bit definitions and macros for EDMA_TCD_DADDR */
#define EDMA_TCD_DADDR_DADDR(x)          (((x)&0xFFFFFFFF)<<0)

/* Bit definitions and macros for EDMA_TCD_CITER */
#define EDMA_TCD_CITER_CITER(x)          (((x)&0x7FFF)<<0)
#define EDMA_TCD_CITER_E_LINK            (0x8000)

/* Bit definitions and macros for EDMA_TCD_CITER_ELINK */
#define EDMA_TCD_CITER_ELINK_CITER(x)    (((x)&0x01FF)<<0)
#define EDMA_TCD_CITER_ELINK_LINKCH(x)   (((x)&0x003F)<<9)
#define EDMA_TCD_CITER_ELINK_E_LINK      (0x8000)

/* Bit definitions and macros for EDMA_TCD_DOFF */
#define EDMA_TCD_DOFF_DOFF(x)            (((x)&0xFFFF)<<0)

/* Bit definitions and macros for EDMA_TCD_DLAST_SGA */
#define EDMA_TCD_DLAST_SGA_DLAST_SGA(x)  (((x)&0xFFFFFFFF)<<0)

/* Bit definitions and macros for EDMA_TCD_BITER */
#define EDMA_TCD_BITER_BITER(x)          (((x)&0x7FFF)<<0)
#define EDMA_TCD_BITER_E_LINK            (0x8000)

/* Bit definitions and macros for EDMA_TCD_BITER_ELINK */
#define EDMA_TCD_BITER_ELINK_BITER(x)    (((x)&0x01FF)<<0)
#define EDMA_TCD_BITER_ELINK_LINKCH(x)   (((x)&0x003F)<<9)
#define EDMA_TCD_BITER_ELINK_E_LINK      (0x8000)

/* Bit definitions and macros for EDMA_TCD_CSR */
#define EDMA_TCD_CSR_START               (0x0001)
#define EDMA_TCD_CSR_INT_MAJOR           (0x0002)
#define EDMA_TCD_CSR_INT_HALF            (0x0004)
#define EDMA_TCD_CSR_D_REQ               (0x0008)
#define EDMA_TCD_CSR_E_SG                (0x0010)
#define EDMA_TCD_CSR_E_LINK              (0x0020)
#define EDMA_TCD_CSR_ACTIVE              (0x0040)
#define EDMA_TCD_CSR_DONE                (0x0080)
#define EDMA_TCD_CSR_LINKCH(x)           (((x)&0x003F)<<8)
#define EDMA_TCD_CSR_BWC(x)              (((x)&0x0003)<<14)
#define EDMA_TCD_CSR_BWC_NO_STALL        (0x0000)
#define EDMA_TCD_CSR_BWC_4CYC_STALL      (0x8000)
#define EDMA_TCD_CSR_BWC_8CYC_STALL      (0xC000)

/********************************************************************/
/* DMA MUX definition */

/* Mux 0 */
#define DMA_MUX0_BASEADDR           0x40021000
#define DMA_MUX0_CHCONFIG0          (*(vuint8_t*)(DMA_MUX0_BASEADDR + 0x00000)) 
#define DMA_MUX0_CHCONFIG1          (*(vuint8_t*)(DMA_MUX0_BASEADDR + 0x00001)) 
#define DMA_MUX0_CHCONFIG2          (*(vuint8_t*)(DMA_MUX0_BASEADDR + 0x00002)) 

#define DMA_MUX0_CHCONFIG12         (*(vuint8_t*)(DMA_MUX0_BASEADDR + 0x0000C)) 
#define DMA_MUX0_CHCONFIG13         (*(vuint8_t*)(DMA_MUX0_BASEADDR + 0x0000D)) 
#define DMA_MUX0_CHCONFIG14         (*(vuint8_t*)(DMA_MUX0_BASEADDR + 0x0000E)) 
#define DMA_MUX0_CHCONFIG15         (*(vuint8_t*)(DMA_MUX0_BASEADDR + 0x0000F)) 

/* Mux 1 */
#define DMA_MUX1_BASEADDR           0x40022000
#define DMA_MUX1_CHCONFIG0          (*(vuint8_t*)(DMA_MUX1_BASEADDR + 0x00000)) 
#define DMA_MUX1_CHCONFIG1          (*(vuint8_t*)(DMA_MUX1_BASEADDR + 0x00001)) 
#define DMA_MUX1_CHCONFIG2          (*(vuint8_t*)(DMA_MUX1_BASEADDR + 0x00002)) 

#define DMA_MUX1_CHCONFIG12         (*(vuint8_t*)(DMA_MUX1_BASEADDR + 0x0000C)) 
#define DMA_MUX1_CHCONFIG13         (*(vuint8_t*)(DMA_MUX1_BASEADDR + 0x0000D)) 
#define DMA_MUX1_CHCONFIG14         (*(vuint8_t*)(DMA_MUX1_BASEADDR + 0x0000E)) 
#define DMA_MUX1_CHCONFIG15         (*(vuint8_t*)(DMA_MUX1_BASEADDR + 0x0000F)) 

#endif /* __PIONEER3_EDMA_H__ */
