/*
 * File:		gpio.c
 * Purpose:		LED Example
 *
 *              Configures GPIO for the LED on the SQM4 EASY BOARD with SQM4_K70 
 *              Blue LED - On and toggles on/off
 *
 */


#define GPIO_PIN_MASK            0x1Fu
#define GPIO_PIN(x)              (((1)<<(x & GPIO_PIN_MASK)))

#include "common.h"
#include "lptmr.h"

//Function declarations
void init_gpio(void);

/********************************************************************/

void main (void)
{
	unsigned int counter=0;
	
  printf("\r\n\n***********************************\n");
  printf("\r*** SQM4-K70F120M GPIO Example! ***\n");
  printf("\r***********************************\n");

  /* Turn on all port clocks */
  SIM_SCGC5 = SIM_SCGC5_PORTA_MASK | SIM_SCGC5_PORTB_MASK | SIM_SCGC5_PORTC_MASK | SIM_SCGC5_PORTD_MASK | SIM_SCGC5_PORTE_MASK;

  
  /* Initialize GPIO on SQM4 EASY BOARD-K70F120M */
  init_gpio();

  while(1)
  {
		counter++;
		printf("\rLED Cycle No. %u\n", counter);

    //Set PTD0 to 0 (turns on blue LED)
    GPIOD_PDOR&=~GPIO_PDOR_PDO(GPIO_PIN(0));
		printf("\r     LED toggled ON for 2000ms\n");
    time_delay_ms(2000);

    //Toggle the green LED on PTA29
    GPIOD_PTOR|=GPIO_PDOR_PDO(GPIO_PIN(0));
		printf("\r     LED toggled OFF for 500ms\n");
    time_delay_ms(500);
  }
}

/*                                                                
 * Initialize GPIO for LED
 *

 *   PTD0 - Blue (E4)
 *
 */
void init_gpio()
{
  //Set PTD0 (connected to LED) for GPIO functionality
  PORTD_PCR0=(0|PORT_PCR_MUX(1));

  //Change PTD0 to outputs
  GPIOD_PDDR=GPIO_PDDR_PDD(GPIO_PIN(0));	
}

/********************************************************************/
