/*
 * File:		lcdc.c
 * Purpose:		LCD Controller Example
 *
 * This is an example of using the LCD controller (LCDC) to drive the 
 *   TWR-LCD-RGB display. 
 *
 * Included in this project is code to do the Background Look Up Table (BGLUT)
 *  and Graphic Window Look up Table (GW). These are not used in the 24bpp mode
 *  that the TWR-LCD-RGB uses, but are for example purposes only. 
 *
 */
#include "common.h"
#include "lcdc.h"
#include "lptmr.h"
#include "sdram.h"

#define VERSION 11
//#define VERSION 10

#define GPIO_PIN_MASK            0x1Fu
#define GPIO_PIN(x)              (((1)<<(x & GPIO_PIN_MASK)))

/********************************************************************/
void main (void)
{
  printf("LCD Controller Example\n");

  //Enable Port clocks
  SIM_SCGC5 |= SIM_SCGC5_PORTA_MASK | SIM_SCGC5_PORTB_MASK | SIM_SCGC5_PORTC_MASK | SIM_SCGC5_PORTD_MASK | SIM_SCGC5_PORTE_MASK;

  /* Enable LCDC and DDR Modules */
  SIM_SCGC3 |= SIM_SCGC3_LCDC_MASK;
  
  twr_ddr2_script_init();

  /* Disable MPU */
  MPU_CESR &= ~MPU_CESR_VLD_MASK;

  /* Setup LCD pin muxing */
  lcdc_init_pins();

  /* LCDC configuration for TWR-LCD-RGB */
  lcdc_init_screen();

	// set LCD Screen Start Address
  LCDC_LSSAR = FRAME_START_ADDRESS;    
  SIM_MCR|=SIM_MCR_LCDSTART_MASK;  //Start LCDC      
	
  printf("\r\n\n***************************************\n");
  printf("\r*** SQM4-K70F120M LCD test Example! ***\n");
  printf("\r***************************************\n");

	//color testing demo
	while(1){
		printf("\r     LCD display WHITE color for 2s\n");
		lcdc_test(0x00FFFFFF);
		time_delay_ms(2000);

		printf("\r     LCD display RED color for 2s\n");
		lcdc_test(0x00FF0000);
		time_delay_ms(2000);
		
		printf("\r     LCD display GREEN color for 2s\n");
		lcdc_test(0x0000FF00);
		time_delay_ms(2000);

		printf("\r     LCD display BLUE color for 2s\n");
		lcdc_test(0x000000FF);
		time_delay_ms(2000);

		printf("\r     LCD display YELLOW color for 2s\n");
		lcdc_test(0x00FFFF00);
		time_delay_ms(2000);

		printf("\r     LCD display PURPLE color for 2s\n");
		lcdc_test(0x00FF00FF);
		time_delay_ms(2000);

		printf("\r     LCD display CYAN color for 2s\n");
		lcdc_test(0x0000FFFF);
		time_delay_ms(2000);

		printf("\r     LCD display BLACK color for 2s\n");
		lcdc_test(0x00000000);
		time_delay_ms(2000);

		printf("\r     LCD display PALLETE color for 2s\n");
		lcdc_test(0xFFFFFFFF);
		time_delay_ms(2000);
		printf("\r\rPress Enter to repeate\n");
		in_char();
	}
}

void lcdc_init_pins()
{
  #define ALT1 			(0|PORT_PCR_MUX(1)|PORT_PCR_DSE_MASK)
  #define ALT2 			(0|PORT_PCR_MUX(2)|PORT_PCR_DSE_MASK)
  #define ALT5 			(0|PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK)
  #define ALT7 			(0|PORT_PCR_MUX(7)|PORT_PCR_DSE_MASK)
  
  
  PORTF_PCR4 =ALT7; // Graphic LCD D[0],  Schematic PTF4
  PORTF_PCR5 =ALT7; // Graphic LCD D[1],  Schematic PTF5
  PORTF_PCR6 =ALT7; // Graphic LCD D[2],  Schematic PTF6
  PORTF_PCR7 =ALT7; // Graphic LCD D[3],  Schematic PTF7
  PORTF_PCR8 =ALT7; // Graphic LCD D[4],  Schematic PTF8
  PORTF_PCR9 =ALT7; // Graphic LCD D[5],  Schematic PTF9
  PORTF_PCR10=ALT7; // Graphic LCD D[6],  Schematic PTF10
  PORTF_PCR11=ALT7; // Graphic LCD D[7],  Schematic PTF11
  PORTF_PCR12=ALT7; // Graphic LCD D[8],  Schematic PTF12
  PORTF_PCR13=ALT7; // Graphic LCD D[9],  Schematic PTF13
  PORTF_PCR14=ALT7; // Graphic LCD D[10], Schematic PTF14
  PORTF_PCR15=ALT7; // Graphic LCD D[11], Schematic PTF15

  PORTF_PCR16=ALT5; // Graphic LCD D[12], Schematic PTF16
  PORTF_PCR17=ALT5; // Graphic LCD D[13], Schematic PTF17
  PORTF_PCR18=ALT5; // Graphic LCD D[14], Schematic PTF18
  PORTE_PCR26=ALT5; // Graphic LCD D[15], Schematic PTF19
  PORTE_PCR27=ALT5; // Graphic LCD D[16], Schematic PTF20

  PORTE_PCR28=ALT5; // Graphic LCD D[17], Schematic PTF21
  PORTF_PCR22=ALT7; // Graphic LCD D[18], Schematic PTF22
  PORTF_PCR23=ALT7; // Graphic LCD D[19], Schematic PTF23
  PORTF_PCR24=ALT7; // Graphic LCD D[20], Schematic PTF24
  PORTF_PCR25=ALT7; // Graphic LCD D[21], Schematic PTF25
  PORTF_PCR26=ALT7; // Graphic LCD D[22], Schematic PTF26
  PORTF_PCR27=ALT7; // Graphic LCD D[23], Schematic PTF27

  PORTF_PCR0=ALT7;  // Graphic LCD PCLK,  Schematic PTF0
  PORTF_PCR1=ALT7;  // Graphic LCD DE,    Schematic PTF1
  PORTF_PCR2=ALT7;  // Graphic LCD HSYNC, Schematic PTF2
  PORTF_PCR3=ALT7;  // Graphic LCD VSYNC, Schematic PTF3

  // set LCD_ON
  PORTE_PCR8=ALT1;  // Graphic LCD On/Off,  Schematic PTE8
  GPIOE_PDDR=GPIO_PDDR_PDD(GPIO_PIN(8));	
  GPIOE_PDOR|=GPIO_PDOR_PDO(GPIO_PIN(8));
#if(VERSION==11)
	PORTD_PCR11=ALT1;
	GPIOD_PDDR|=GPIO_PDDR_PDD(GPIO_PIN(11));
	GPIOD_PDOR|=GPIO_PDOR_PDO(GPIO_PIN(11));
#endif	
}

/********************************************************************/

/*
 * Set up the LCDC for the LCD screen on the TWR-LCD-RGB
 */
void lcdc_init_screen()
{
  // set LCD Screen Start Address
  LCDC_LSSAR = FRAME_START_ADDRESS;  

  // set LCD Size. The XMAX bitfield is the screen x-size/16. 
  LCDC_LSR = LCDC_LSR_XMAX( (SCREEN_XSIZE / 16) ) | LCDC_LSR_YMAX( SCREEN_YSIZE );

  // set LCD virtual page width
  LCDC_LVPWR = LCDC_LVPWR_VPW( SCREEN_XSIZE );

  // set LCD cursor positon & settings (turn off)
  LCDC_LCPR = 0;
  LCDC_LCWHB = 0;

  // set LCD panel configuration. Use endianess to work with TWR-LCD-RGB lines.
  LCDC_LPCR =
    LCDC_LPCR_TFT_MASK      |       //TFT Screen
    LCDC_LPCR_COLOR_MASK    |       //Color
    LCDC_LPCR_BPIX(0x7)     |       //24 bpp
    LCDC_LPCR_FLMPOL_MASK   |       //first line marker active low (VSYNC)
    LCDC_LPCR_LPPOL_MASK    |       //line pulse active low (HSYNC)
    LCDC_LPCR_END_SEL_MASK  |       //Use big-endian mode (0xFFAA5500 means R=AA,G=55,B=00).     
    //LCDC_LPCR_SWAP_SEL_MASK |       //Set so that the LCD data lines match up correctly with the TWR-LCD-RGB
    LCDC_LPCR_SCLKIDLE_MASK |       //Enalbe LSCLK when vsync is idle
    LCDC_LPCR_SCLKSEL_MASK  |       //Always enable clock
    LCDC_LPCR_PCD(11);             //Divide 120 PLL0 clock (default clock) by (11+1)=12 to get 10MHz clock

  // set LCD horizontal configuration based on panel data (Figure 3-3 in Seiko datasheet)
#if defined(FRD5040TPT) || defined(FRD7040TPT) 
  LCDC_LHCR =
    LCDC_LHCR_H_WIDTH(47)   |    //(47+1)=48 SCLK period for HSYNC activated
    LCDC_LHCR_H_WAIT_1(39)  |    //(39+1)=40 SCLK period between end of OE and beginning of HSYNC
    LCDC_LHCR_H_WAIT_2(85);     //(85+3)=88 SCLK periods between end of HSYNC and beginning of OE

  // set LCD vertical configuration based on panel data (Figure 3-3 in Seiko datasheet)
  LCDC_LVCR =
    LCDC_LVCR_V_WIDTH(3)  |   //3 lines period for VSYNC activated
    LCDC_LVCR_V_WAIT_1(13) |    //13 line period between end of OE and beginning of VSYNC
    LCDC_LVCR_V_WAIT_2(32);    //32 line periods between end of VSYNC and beginning of OE
#else
  LCDC_LHCR =
    LCDC_LHCR_H_WIDTH(40)   |    //(40+1)=41 SCLK period for HSYNC activated
    LCDC_LHCR_H_WAIT_1(1)  |    //(1+1)=2 SCLK period between end of OE and beginning of HSYNC
    LCDC_LHCR_H_WAIT_2(0);     //(0+3)=3 SCLK periods between end of HSYNC and beginning of OE

  // set LCD vertical configuration based on panel data (Figure 3-3 in Seiko datasheet)
  LCDC_LVCR =
    LCDC_LVCR_V_WIDTH(10)  |   //10 lines period for VSYNC activated
    LCDC_LVCR_V_WAIT_1(2) |    //2 line period between end of OE and beginning of VSYNC
    LCDC_LVCR_V_WAIT_2(2);    //2 line periods between end of VSYNC and beginning of OE
#endif
  // set the LCD panning offset (not used in 24bpp mode)
  LCDC_LPOR = 0;

  // set LCD interrupt configuration register
  LCDC_LICR = 0;

  //Disable LCDC interrupts
  LCDC_LIER = 0;

  //Disable the graphic window. See the "color" and "fsl" demos for examples of
  //  using the graphic window feature
  LCDC_LGWCR &=~LCDC_LGWCR_GWE_MASK;
  
  //Set background plane DMA to burst mode
  LCDC_LDCR&=~LCDC_LDCR_BURST_MASK;  
  
  //Set graphic window DMA to burst mode
  LCDC_LGWDCR&=~LCDC_LGWDCR_GWBT_MASK;  
}

void lcdc_test(unsigned int color){
  int *pointer32;
  int i,n;  
  pointer32=(int *)FRAME_START_ADDRESS;

	if(color!=0xFFFFFFFF){                  //Loop through screen
		for(i=0;i<SCREEN_YSIZE;i++){
			for(n=0;n<SCREEN_XSIZE;n++){
				*pointer32=color; 
				pointer32++;      
			}
		}
	}
	else{
		for(i=0;i<SCREEN_YSIZE/2;i++){
			for(n=0;n<SCREEN_XSIZE/4;n++){       //One quarter will be red
				*pointer32=0x00FF0000;             //Red
				pointer32++;      
			}
			for(n=0;n<SCREEN_XSIZE/4;n++){
				*pointer32=0x00FFFF00; //Yellow
				pointer32++;      
			}
			for(n=0;n<SCREEN_XSIZE/4;n++){        //One quarter will be green
				*pointer32=0x0000C000; //Green
				pointer32++;      
			}    
			for(n=0;n<SCREEN_XSIZE/4;n++){
				*pointer32=0x0000A0A0; //Azur
				pointer32++;      
			}
		}
		for(i=0;i<SCREEN_YSIZE/2;i++){          //Loop through bottom half of screen  
			for(n=0;n<SCREEN_XSIZE/4;n++){        //One quarter will be blue
				*pointer32=0x000000FF; //Blue
				pointer32++;      
			}
			for(n=0;n<SCREEN_XSIZE/4;n++){
				*pointer32=0x00B000B0; //Purple
				pointer32++;      
			}
			for(n=0;n<SCREEN_XSIZE/4;n++){        //One quarter will be white
				*pointer32=0x00FFFFFF; //White
				pointer32++;      
			}    
			for(n=0;n<SCREEN_XSIZE/4;n++){
				*pointer32=0x00808080; //Gray
				pointer32++;      
			}
		}
	}
}

